@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@
  */
 
 #include <string.h>
+#include <dbus/dbus.h>
 #include <ibusinternal.h>
 #include <ibusmarshalers.h>
 #include "dbusimpl.h"
@@ -40,8 +41,6 @@ enum {
 static guint dbus_signals[LAST_SIGNAL] = { 0 };
 
 /* functions prototype */
-static void     bus_dbus_impl_class_init        (BusDBusImplClass   *klass);
-static void     bus_dbus_impl_init              (BusDBusImpl        *dbus);
 static void     bus_dbus_impl_destroy           (BusDBusImpl        *dbus);
 static gboolean bus_dbus_impl_ibus_message      (BusDBusImpl        *dbus,
                                                  BusConnection      *connection,
@@ -57,33 +56,7 @@ static void     _connection_destroy_cb          (BusConnection      *connection,
 static void     _rule_destroy_cb                (BusMatchRule       *rule,
                                                  BusDBusImpl        *dbus);
 
-static IBusServiceClass  *parent_class = NULL;
-
-GType
-bus_dbus_impl_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (BusDBusImplClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    bus_dbus_impl_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (BusDBusImpl),
-        0,
-        (GInstanceInitFunc) bus_dbus_impl_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_SERVICE,
-                    "BusDBusImpl",
-                    &type_info,
-                    (GTypeFlags) 0);
-    }
-    return type;
-}
+G_DEFINE_TYPE(BusDBusImpl, bus_dbus_impl, IBUS_TYPE_SERVICE)
 
 BusDBusImpl *
 bus_dbus_impl_get_default (void)
@@ -105,8 +78,6 @@ bus_dbus_impl_class_init (BusDBusImplClass *klass)
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
     IBusServiceClass *service_class = IBUS_SERVICE_CLASS (klass);
 
-    parent_class = (IBusServiceClass *) g_type_class_peek_parent (klass);
-
     IBUS_OBJECT_CLASS (gobject_class)->destroy = (IBusObjectDestroyFunc) bus_dbus_impl_destroy;
 
     service_class->ibus_message = (ServiceIBusMessageFunc) bus_dbus_impl_ibus_message;
@@ -175,7 +146,7 @@ bus_dbus_impl_destroy (BusDBusImpl *dbus)
     dbus->names = NULL;
     dbus->objects = NULL;
 
-    G_OBJECT_CLASS(parent_class)->dispose (G_OBJECT (dbus));
+    G_OBJECT_CLASS(bus_dbus_impl_parent_class)->dispose (G_OBJECT (dbus));
 }
 
 
@@ -778,9 +749,10 @@ bus_dbus_impl_ibus_message (BusDBusImpl  *dbus,
         }
     }
 
-    return parent_class->ibus_message ((IBusService *) dbus,
-                                       (IBusConnection *) connection,
-                                       message);
+    return IBUS_SERVICE_CLASS (bus_dbus_impl_parent_class)->ibus_message (
+                                (IBusService *) dbus,
+                                (IBusConnection *) connection,
+                                message);
 }
 
 static void
@@ -979,7 +951,7 @@ bus_dbus_impl_new_connection (BusDBusImpl    *dbus,
 
     g_assert (g_list_find (dbus->connections, connection) == NULL);
 
-    g_object_ref (connection);
+    g_object_ref_sink (connection);
     dbus->connections = g_list_append (dbus->connections, connection);
 
     g_signal_connect (connection,
@@ -1100,7 +1072,6 @@ bus_dbus_impl_dispatch_message_by_rule (BusDBusImpl     *dbus,
         if (connection != skip_connection) {
             ibus_connection_send (IBUS_CONNECTION (connection), message);
         }
-        g_object_unref (connection);
     }
     g_list_free (recipients);
 }
@@ -1132,7 +1103,7 @@ bus_dbus_impl_register_object (BusDBusImpl *dbus,
 
     g_return_val_if_fail  (g_hash_table_lookup (dbus->objects, path) == NULL, FALSE);
 
-    g_object_ref (object);
+    g_object_ref_sink (object);
     g_hash_table_insert (dbus->objects, (gpointer)path, object);
 
     g_signal_connect (object, "destroy", G_CALLBACK (_object_destroy_cb), dbus);
