@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+#include <dbus/dbus.h>
 #include <ibusinternal.h>
 #include <ibusmarshalers.h>
 #include "ibusimpl.h"
@@ -52,40 +52,12 @@ static guint    engine_signals[LAST_SIGNAL] = { 0 };
 // static guint            engine_signals[LAST_SIGNAL] = { 0 };
 
 /* functions prototype */
-static void     bus_engine_proxy_class_init     (BusEngineProxyClass    *klass);
-static void     bus_engine_proxy_init           (BusEngineProxy         *engine);
 static void     bus_engine_proxy_real_destroy   (BusEngineProxy         *engine);
 
 static gboolean bus_engine_proxy_ibus_signal    (IBusProxy              *proxy,
                                                  IBusMessage            *message);
 
-static IBusProxyClass  *parent_class = NULL;
-
-GType
-bus_engine_proxy_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (BusEngineProxyClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    bus_engine_proxy_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (BusEngineProxy),
-        0,
-        (GInstanceInitFunc) bus_engine_proxy_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_PROXY,
-                    "BusEngineProxy",
-                    &type_info,
-                    (GTypeFlags)0);
-    }
-    return type;
-}
+G_DEFINE_TYPE (BusEngineProxy, bus_engine_proxy, IBUS_TYPE_PROXY)
 
 BusEngineProxy *
 bus_engine_proxy_new (const gchar    *path,
@@ -105,13 +77,13 @@ bus_engine_proxy_new (const gchar    *path,
                                               NULL);
 
     engine->desc = desc;
-    g_object_ref (desc);
+    g_object_ref_sink (desc);
     if (desc->layout != NULL && desc->layout[0] != '\0') {
-        engine->keymap = ibus_keymap_new (desc->layout);
+        engine->keymap = ibus_keymap_get (desc->layout);
     }
 
     if (engine->keymap == NULL) {
-        engine->keymap = ibus_keymap_new ("us");
+        engine->keymap = ibus_keymap_get ("us");
     }
 
     return engine;
@@ -124,7 +96,7 @@ bus_engine_proxy_class_init (BusEngineProxyClass *klass)
     IBusProxyClass *proxy_class = IBUS_PROXY_CLASS (klass);
 
 
-    parent_class = (IBusProxyClass *) g_type_class_peek_parent (klass);
+    bus_engine_proxy_parent_class = (IBusProxyClass *) g_type_class_peek_parent (klass);
 
     ibus_object_class->destroy = (IBusObjectDestroyFunc) bus_engine_proxy_real_destroy;
 
@@ -173,12 +145,13 @@ bus_engine_proxy_class_init (BusEngineProxyClass *klass)
             G_SIGNAL_RUN_LAST,
             0,
             NULL, NULL,
-            ibus_marshal_VOID__OBJECT_UINT_BOOLEAN,
+            ibus_marshal_VOID__OBJECT_UINT_BOOLEAN_UINT,
             G_TYPE_NONE,
-            3,
+            4,
             IBUS_TYPE_TEXT,
             G_TYPE_UINT,
-            G_TYPE_BOOLEAN);
+            G_TYPE_BOOLEAN,
+            G_TYPE_UINT);
 
     engine_signals[SHOW_PREEDIT_TEXT] =
         g_signal_new (I_("show-preedit-text"),
@@ -331,8 +304,13 @@ bus_engine_proxy_class_init (BusEngineProxyClass *klass)
 static void
 bus_engine_proxy_init (BusEngineProxy *engine)
 {
+    engine->has_focus = FALSE;
+    engine->enabled = FALSE;
+    engine->x = 0;
+    engine->y = 0;
+    engine->w = 0;
+    engine->h = 0;
     engine->enabled = FALSE;
-    engine->prop_list = NULL;
     engine->desc = NULL;
     engine->keymap = NULL;
 }
@@ -340,11 +318,6 @@ bus_engine_proxy_init (BusEngineProxy *engine)
 static void
 bus_engine_proxy_real_destroy (BusEngineProxy *engine)
 {
-    if (engine->prop_list) {
-        g_object_unref (engine->prop_list);
-        engine->prop_list = NULL;
-    }
-
     if (ibus_proxy_get_connection ((IBusProxy *) engine)) {
         ibus_proxy_call ((IBusProxy *) engine,
                          "Destroy",
@@ -361,7 +334,7 @@ bus_engine_proxy_real_destroy (BusEngineProxy *engine)
         engine->keymap = NULL;
     }
 
-    IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (engine));
+    IBUS_OBJECT_CLASS(bus_engine_proxy_parent_class)->destroy (IBUS_OBJECT (engine));
 }
 
 static gboolean
@@ -463,20 +436,23 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
         gint cursor_pos;
         gboolean visible;
         gboolean retval;
+        guint mode;
 
         retval = ibus_message_get_args (message,
                                         &error,
                                         IBUS_TYPE_TEXT, &text,
                                         G_TYPE_UINT, &cursor_pos,
                                         G_TYPE_BOOLEAN, &visible,
+                                        G_TYPE_UINT, &mode,
                                         G_TYPE_INVALID);
 
         if (!retval)
             goto failed;
 
         g_signal_emit (engine, engine_signals[UPDATE_PREEDIT_TEXT], 0,
-                       text, cursor_pos, visible);
-        g_object_unref (text);
+                        text, cursor_pos, visible, mode);
+        if (g_object_is_floating (text))
+            g_object_unref (text);
     }
     else if (g_strcmp0 (name, "UpdateAuxiliaryText") == 0) {
         IBusText *text;
@@ -493,7 +469,8 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
             goto failed;
 
         g_signal_emit (engine, engine_signals[UPDATE_AUXILIARY_TEXT], 0, text, visible);
-        g_object_unref (text);
+        if (g_object_is_floating (text))
+            g_object_unref (text);
     }
     else if (g_strcmp0 (name, "UpdateLookupTable") == 0) {
         IBusLookupTable *table;
@@ -510,25 +487,25 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
             goto failed;
 
         g_signal_emit (engine, engine_signals[UPDATE_LOOKUP_TABLE], 0, table, visible);
-        g_object_unref (table);
+        if (g_object_is_floating (table))
+            g_object_unref (table);
     }
     else if (g_strcmp0 (name, "RegisterProperties") == 0) {
         gboolean retval;
-
-        if (engine->prop_list) {
-            g_object_unref (engine->prop_list);
-            engine->prop_list = NULL;
-        }
+        IBusPropList *prop_list;
 
         retval = ibus_message_get_args (message,
                                         &error,
-                                        IBUS_TYPE_PROP_LIST, &engine->prop_list,
+                                        IBUS_TYPE_PROP_LIST, &prop_list,
                                         G_TYPE_INVALID);
         if (!retval) {
-            engine->prop_list = NULL;
             goto failed;
         }
-        g_signal_emit (engine, engine_signals[REGISTER_PROPERTIES], 0, engine->prop_list);
+        g_signal_emit (engine, engine_signals[REGISTER_PROPERTIES], 0, prop_list);
+
+        if (g_object_is_floating (prop_list))
+            g_object_unref (prop_list);
+
     }
     else if (g_strcmp0 (name, "UpdateProperty") == 0) {
         IBusProperty *prop;
@@ -543,7 +520,8 @@ bus_engine_proxy_ibus_signal (IBusProxy     *proxy,
             goto failed;
 
         g_signal_emit (engine, engine_signals[UPDATE_PROPERTY], 0, prop);
-        g_object_unref (prop);
+        if (g_object_is_floating (prop))
+            g_object_unref (prop);
     }
     else
         return FALSE;
@@ -680,13 +658,19 @@ bus_engine_proxy_set_cursor_location (BusEngineProxy *engine,
 {
     g_assert (BUS_IS_ENGINE_PROXY (engine));
 
-    ibus_proxy_call ((IBusProxy *) engine,
-                     "SetCursorLocation",
-                     G_TYPE_INT, &x,
-                     G_TYPE_INT, &y,
-                     G_TYPE_INT, &w,
-                     G_TYPE_INT, &h,
-                     G_TYPE_INVALID);
+    if (engine->x != x || engine->y != y || engine->w != w || engine->h != h) {
+        engine->x = x;
+        engine->y = y;
+        engine->w = w;
+        engine->h = h;
+        ibus_proxy_call ((IBusProxy *) engine,
+                         "SetCursorLocation",
+                         G_TYPE_INT, &x,
+                         G_TYPE_INT, &y,
+                         G_TYPE_INT, &w,
+                         G_TYPE_INT, &h,
+                         G_TYPE_INVALID);
+    }
 }
 
 void
@@ -695,11 +679,13 @@ bus_engine_proxy_set_capabilities (BusEngineProxy *engine,
 {
     g_assert (BUS_IS_ENGINE_PROXY (engine));
 
-    ibus_proxy_call ((IBusProxy *) engine,
-                     "SetCapabilities",
-                     G_TYPE_UINT, &caps,
-                     G_TYPE_INVALID);
-
+    if (engine->capabilities != caps) {
+        engine->capabilities = caps;
+        ibus_proxy_call ((IBusProxy *) engine,
+                         "SetCapabilities",
+                         G_TYPE_UINT, &caps,
+                         G_TYPE_INVALID);
+    }
 }
 
 void
@@ -752,18 +738,62 @@ void bus_engine_proxy_property_hide (BusEngineProxy *engine,
                      G_TYPE_INVALID);                       \
     }
 
-DEFINE_FUNCTION (FocusIn, focus_in)
-DEFINE_FUNCTION (FocusOut, focus_out)
 DEFINE_FUNCTION (Reset, reset)
 DEFINE_FUNCTION (PageUp, page_up)
 DEFINE_FUNCTION (PageDown, page_down)
 DEFINE_FUNCTION (CursorUp, cursor_up)
 DEFINE_FUNCTION (CursorDown, cursor_down)
-DEFINE_FUNCTION (Enable, enable)
-DEFINE_FUNCTION (Disable, disable)
 
 #undef DEFINE_FUNCTION
 
+void
+bus_engine_proxy_focus_in (BusEngineProxy *engine)
+{
+    g_assert (BUS_IS_ENGINE_PROXY (engine));
+    if (!engine->has_focus) {
+        engine->has_focus = TRUE;
+        ibus_proxy_call ((IBusProxy *) engine,
+                         "FocusIn",
+                         G_TYPE_INVALID);
+    }
+}
+
+void
+bus_engine_proxy_focus_out (BusEngineProxy *engine)
+{
+    g_assert (BUS_IS_ENGINE_PROXY (engine));
+    if (engine->has_focus) {
+        engine->has_focus = FALSE;
+        ibus_proxy_call ((IBusProxy *) engine,
+                         "FocusOut",
+                         G_TYPE_INVALID);
+    }
+}
+
+void
+bus_engine_proxy_enable (BusEngineProxy *engine)
+{
+    g_assert (BUS_IS_ENGINE_PROXY (engine));
+    if (!engine->enabled) {
+        engine->enabled = TRUE;
+        ibus_proxy_call ((IBusProxy *) engine,
+                         "Enable",
+                         G_TYPE_INVALID);
+    }
+}
+
+void
+bus_engine_proxy_disable (BusEngineProxy *engine)
+{
+    g_assert (BUS_IS_ENGINE_PROXY (engine));
+    if (engine->enabled) {
+        engine->enabled = FALSE;
+        ibus_proxy_call ((IBusProxy *) engine,
+                         "Disable",
+                         G_TYPE_INVALID);
+    }
+}
+
 void
 bus_engine_proxy_candidate_clicked (BusEngineProxy *engine,
                                     guint           index,
@@ -787,3 +817,11 @@ bus_engine_proxy_get_desc (BusEngineProxy *engine)
 
     return engine->desc;
 }
+
+gboolean
+bus_engine_proxy_is_enabled (BusEngineProxy *engine)
+{
+    g_assert (BUS_IS_ENGINE_PROXY (engine));
+
+    return engine->enabled;
+}
