@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+#include <dbus/dbus.h>
 #include <ibusinternal.h>
 #include <ibusmarshalers.h>
 #include "dbusimpl.h"
@@ -26,38 +26,9 @@
 #include "option.h"
 
 /* functions prototype */
-static void      bus_factory_proxy_class_init   (BusFactoryProxyClass   *klass);
-static void      bus_factory_proxy_init         (BusFactoryProxy        *factory);
 static void      bus_factory_proxy_destroy      (BusFactoryProxy        *factory);
 
-
-static IBusProxyClass  *parent_class = NULL;
-
-GType
-bus_factory_proxy_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (BusFactoryProxyClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    bus_factory_proxy_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (BusFactoryProxy),
-        0,
-        (GInstanceInitFunc) bus_factory_proxy_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_PROXY,
-                    "BusFactoryProxy",
-                    &type_info,
-                    (GTypeFlags)0);
-    }
-    return type;
-}
+G_DEFINE_TYPE (BusFactoryProxy, bus_factory_proxy, IBUS_TYPE_PROXY)
 
 BusFactoryProxy *
 bus_factory_proxy_new (IBusComponent *component,
@@ -82,7 +53,7 @@ bus_factory_proxy_new (IBusComponent *component,
                             "connection", connection,
                             NULL);
 
-    g_object_ref (component);
+    g_object_ref_sink (component);
     factory->component = component;
     g_object_set_data ((GObject *)factory->component, "factory", factory);
 
@@ -102,10 +73,7 @@ bus_factory_proxy_class_init (BusFactoryProxyClass *klass)
 {
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (klass);
 
-    parent_class = (IBusProxyClass *) g_type_class_peek_parent (klass);
-
     ibus_object_class->destroy = (IBusObjectDestroyFunc) bus_factory_proxy_destroy;
-
 }
 
 static void
@@ -133,7 +101,7 @@ bus_factory_proxy_destroy (BusFactoryProxy *factory)
         factory->component = NULL;
     }
 
-    IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (factory));
+    IBUS_OBJECT_CLASS(bus_factory_proxy_parent_class)->destroy (IBUS_OBJECT (factory));
 }
 
 IBusComponent *
