@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* bus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,7 @@
 #include "registry.h"
 #include "factoryproxy.h"
 #include "panelproxy.h"
+#include "engineproxy.h"
 
 /*
  * Type macros.
@@ -73,6 +74,8 @@ struct _BusIBusImpl {
     GList *component_list;
 
     gboolean use_sys_layout;
+    gboolean embed_preedit_text;
+    gboolean enable_by_default;
 
     BusRegistry     *registry;
 
@@ -82,6 +85,12 @@ struct _BusIBusImpl {
     IBusHotkeyProfile *hotkey_profile;
     IBusKeymap      *keymap;
 
+    gboolean use_global_engine;
+    BusEngineProxy  *global_engine;
+    gchar           *global_previous_engine_name;
+
+    IBusHotkeyProfile *engines_hotkey_profile;
+    GHashTable      *hotkey_to_engines_map;
 };
 
 struct _BusIBusImplClass {
@@ -104,6 +113,13 @@ IBusHotkeyProfile
 IBusKeymap      *bus_ibus_impl_get_keymap           (BusIBusImpl        *ibus);
 BusRegistry     *bus_ibus_impl_get_registry         (BusIBusImpl        *ibus);
 
+gboolean         bus_ibus_impl_filter_keyboard_shortcuts
+                                                    (BusIBusImpl        *ibus,
+                                                     BusInputContext    *context,
+                                                     guint               keyval,
+                                                     guint               modifiers,
+                                                     guint               prev_keyval,
+                                                     guint               prev_modifiers);
+
 G_END_DECLS
 #endif
-
