@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+#include <dbus/dbus.h>
 #include <ibusinternal.h>
 #include <ibusmarshalers.h>
 #include "ibusimpl.h"
@@ -50,8 +50,6 @@ enum {
     DISABLED,
     ENGINE_CHANGED,
     REQUEST_ENGINE,
-    REQUEST_NEXT_ENGINE,
-    REQUEST_PREV_ENGINE,
     LAST_SIGNAL,
 };
 
@@ -64,8 +62,6 @@ typedef struct _BusInputContextPrivate BusInputContextPrivate;
 static guint    context_signals[LAST_SIGNAL] = { 0 };
 
 /* functions prototype */
-static void     bus_input_context_class_init    (BusInputContextClass   *klass);
-static void     bus_input_context_init          (BusInputContext        *context);
 static void     bus_input_context_destroy       (BusInputContext        *context);
 static gboolean bus_input_context_ibus_message  (BusInputContext        *context,
                                                  BusConnection          *connection,
@@ -81,11 +77,14 @@ static gboolean bus_input_context_send_signal   (BusInputContext        *context
                                                  ...);
 
 static void     bus_input_context_unset_engine  (BusInputContext        *context);
+static void     bus_input_context_commit_text   (BusInputContext        *context,
+                                                 IBusText               *text);
 static void     bus_input_context_update_preedit_text
                                                 (BusInputContext        *context,
                                                  IBusText               *text,
                                                  guint                   cursor_pos,
-                                                 gboolean                visible);
+                                                 gboolean                visible,
+                                                 guint                   mode);
 static void     bus_input_context_show_preedit_text
                                                 (BusInputContext        *context);
 static void     bus_input_context_hide_preedit_text
@@ -123,37 +122,17 @@ static void     bus_input_context_update_property
 static void     _engine_destroy_cb              (BusEngineProxy         *factory,
                                                  BusInputContext        *context);
 
-static IBusServiceClass  *parent_class = NULL;
 static guint id = 0;
 static IBusText *text_empty = NULL;
 static IBusLookupTable *lookup_table_empty = NULL;
 static IBusPropList    *props_empty = NULL;
 
-GType
-bus_input_context_get_type (void)
-{
-    static GType type = 0;
+G_DEFINE_TYPE (BusInputContext, bus_input_context, IBUS_TYPE_SERVICE)
 
-    static const GTypeInfo type_info = {
-        sizeof (BusInputContextClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    bus_input_context_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (BusInputContext),
-        0,
-        (GInstanceInitFunc) bus_input_context_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_SERVICE,
-                    "BusInputContext",
-                    &type_info,
-                    (GTypeFlags) 0);
-    }
-    return type;
-}
+/* when send preedit to client */
+#define PREEDIT_CONDITION  \
+    ((context->capabilities & IBUS_CAP_PREEDIT_TEXT) && \
+    (BUS_DEFAULT_IBUS->embed_preedit_text || (context->capabilities & IBUS_CAP_FOCUS) == 0))
 
 static void
 _connection_destroy_cb (BusConnection   *connection,
@@ -188,10 +167,13 @@ bus_input_context_new (BusConnection    *connection,
                                  IBUS_CONNECTION (connection));
 #endif
 
-    g_object_ref (connection);
+    g_object_ref_sink (connection);
     context->connection = connection;
     context->client = g_strdup (client);
 
+    /* it is a fake input context, just need process hotkey */
+    context->fake = (g_strcmp0 (client, "fake") == 0);
+
     g_signal_connect (context->connection,
                       "destroy",
                       (GCallback) _connection_destroy_cb,
@@ -205,11 +187,10 @@ bus_input_context_class_init (BusInputContextClass *klass)
 {
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (klass);
 
-    parent_class = (IBusServiceClass *) g_type_class_peek_parent (klass);
-
     ibus_object_class->destroy = (IBusObjectDestroyFunc) bus_input_context_destroy;
 
-    IBUS_SERVICE_CLASS (klass)->ibus_message = (ServiceIBusMessageFunc) bus_input_context_ibus_message;
+    IBUS_SERVICE_CLASS (klass)->ibus_message =
+            (ServiceIBusMessageFunc) bus_input_context_ibus_message;
 
     /* install signals */
     context_signals[PROCESS_KEY_EVENT] =
@@ -430,7 +411,7 @@ bus_input_context_class_init (BusInputContextClass *klass)
             0);
 
     context_signals[ENGINE_CHANGED] =
-        g_signal_new (I_("factory-changed"),
+        g_signal_new (I_("engine-changed"),
             G_TYPE_FROM_CLASS (klass),
             G_SIGNAL_RUN_LAST,
             0,
@@ -450,29 +431,12 @@ bus_input_context_class_init (BusInputContextClass *klass)
             1,
             G_TYPE_STRING);
 
-    context_signals[REQUEST_NEXT_ENGINE] =
-        g_signal_new (I_("request-next-engine"),
-            G_TYPE_FROM_CLASS (klass),
-            G_SIGNAL_RUN_LAST,
-            0,
-            NULL, NULL,
-            ibus_marshal_VOID__VOID,
-            G_TYPE_NONE,
-            0);
-
-    context_signals[REQUEST_PREV_ENGINE] =
-        g_signal_new (I_("request-prev-engine"),
-            G_TYPE_FROM_CLASS (klass),
-            G_SIGNAL_RUN_LAST,
-            0,
-            NULL, NULL,
-            ibus_marshal_VOID__VOID,
-            G_TYPE_NONE,
-            0);
-
     text_empty = ibus_text_new_from_string ("");
+    g_object_ref_sink (text_empty);
     lookup_table_empty = ibus_lookup_table_new (9, 0, FALSE, FALSE);
+    g_object_ref_sink (lookup_table_empty);
     props_empty = ibus_prop_list_new ();
+    g_object_ref_sink (props_empty);
 }
 
 static void
@@ -494,20 +458,20 @@ bus_input_context_init (BusInputContext *context)
     context->w = 0;
     context->h = 0;
 
-    g_object_ref (text_empty);
+    g_object_ref_sink (text_empty);
     context->preedit_text = text_empty;
     context->preedit_cursor_pos = 0;
     context->preedit_visible = FALSE;
+    context->preedit_mode = IBUS_ENGINE_PREEDIT_CLEAR;
 
-    g_object_ref (text_empty);
+    g_object_ref_sink (text_empty);
     context->auxiliary_text = text_empty;
     context->auxiliary_visible = FALSE;
 
-    g_object_ref (lookup_table_empty);
+    g_object_ref_sink (lookup_table_empty);
     context->lookup_table = lookup_table_empty;
     context->lookup_table_visible = FALSE;
 
-    g_object_ref (props_empty);
 }
 
 static void
@@ -550,7 +514,7 @@ bus_input_context_destroy (BusInputContext *context)
         context->client = NULL;
     }
 
-    IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (context));
+    IBUS_OBJECT_CLASS(bus_input_context_parent_class)->destroy (IBUS_OBJECT (context));
 }
 
 /* introspectable interface */
@@ -593,6 +557,10 @@ _ibus_introspect (BusInputContext   *context,
         "    <method name=\"SetCapabilities\">\n"
         "      <arg name=\"caps\" direction=\"in\" type=\"u\"/>\n"
         "    </method>\n"
+        "    <method name=\"PropertyActivate\">\n"
+        "      <arg name=\"name\" direction=\"in\" type=\"s\"/>\n"
+        "      <arg name=\"state\" direction=\"in\" type=\"i\"/>\n"
+        "    </method>\n"
         "    <method name=\"SetEngine\">\n"
         "      <arg name=\"name\" direction=\"in\" type=\"s\"/>\n"
         "    </method>\n"
@@ -669,21 +637,24 @@ _ic_process_key_event_reply_cb (gpointer data,
 {
     gboolean retval;
     CallData *call_data;
-    IBusMessage *reply;
 
     retval = (gboolean) GPOINTER_TO_INT (data);
     call_data = (CallData *) user_data;
 
-    reply = ibus_message_new_method_return (call_data->message);
-    ibus_message_append_args (reply,
-                              G_TYPE_BOOLEAN, &retval,
-                              G_TYPE_INVALID);
+    /* make sure the connection is alive */
+    if (G_LIKELY (call_data->context->connection != NULL)) {
+        IBusMessage *reply;
+        reply = ibus_message_new_method_return (call_data->message);
+        ibus_message_append_args (reply,
+                                  G_TYPE_BOOLEAN, &retval,
+                                  G_TYPE_INVALID);
 
-    ibus_connection_send ((IBusConnection *)call_data->context->connection, reply);
+        ibus_connection_send ((IBusConnection *)call_data->context->connection, reply);
+        ibus_message_unref (reply);
+    }
 
     g_object_unref (call_data->context);
     ibus_message_unref (call_data->message);
-    ibus_message_unref (reply);
     g_slice_free (CallData, call_data);
 }
 
@@ -696,7 +667,7 @@ _ic_process_key_event  (BusInputContext *context,
     g_assert (message != NULL);
     g_assert (BUS_IS_CONNECTION (connection));
 
-    IBusMessage *reply = NULL;
+    IBusMessage *reply;
     guint keyval, keycode, modifiers;
     gboolean retval;
     IBusError *error;
@@ -716,15 +687,30 @@ _ic_process_key_event  (BusInputContext *context,
         return reply;
     }
 
-    retval = bus_input_context_filter_keyboard_shortcuts (context, keyval, keycode, modifiers);
-
-    if (retval) {
-        reply = ibus_message_new_method_return (message);
-        ibus_message_append_args (reply,
-                                  G_TYPE_BOOLEAN, &retval,
-                                  G_TYPE_INVALID);
+    if (G_UNLIKELY (!context->has_focus)) {
+        /* workaround: set focus if context does not have focus */
+        if (BUS_DEFAULT_IBUS->focused_context == NULL ||
+            BUS_DEFAULT_IBUS->focused_context->fake == TRUE ||
+            context->fake == FALSE) {
+            /* grab focus, if context is a real IC or current focused IC is fake */
+            bus_input_context_focus_in (context);
+        }
     }
-    else if (context->enabled && context->engine) {
+
+    if (G_LIKELY (context->has_focus)) {
+        retval = bus_input_context_filter_keyboard_shortcuts (context, keyval, keycode, modifiers);
+        /* If it is keyboard shortcut, reply TRUE to client */
+        if (G_UNLIKELY (retval)) {
+            reply = ibus_message_new_method_return (message);
+            ibus_message_append_args (reply,
+                                      G_TYPE_BOOLEAN, &retval,
+                                      G_TYPE_INVALID);
+            return reply;
+        }
+    }
+
+    /* ignore key events, if it is a fake input context */
+    if (context->has_focus && context->enabled && context->engine && context->fake == FALSE) {
         CallData *call_data;
 
         call_data = g_slice_new (CallData);
@@ -741,14 +727,16 @@ _ic_process_key_event  (BusInputContext *context,
                                             modifiers,
                                             (GFunc) _ic_process_key_event_reply_cb,
                                             call_data);
+        return NULL;
     }
     else {
+        retval = FALSE;
         reply = ibus_message_new_method_return (message);
         ibus_message_append_args (reply,
                                   G_TYPE_BOOLEAN, &retval,
                                   G_TYPE_INVALID);
+        return reply;
     }
-    return reply;
 }
 
 static IBusMessage *
@@ -785,7 +773,7 @@ _ic_set_cursor_location (BusInputContext  *context,
     context->h = h;
     context->w = w;
 
-    if (context->engine) {
+    if (context->has_focus && context->enabled && context->engine) {
         bus_engine_proxy_set_cursor_location (context->engine, x, y, w, h);
     }
 
@@ -814,9 +802,15 @@ _ic_focus_in (BusInputContext  *context,
 
     IBusMessage *reply;
 
-    bus_input_context_focus_in (context);
-
-    reply = ibus_message_new_method_return (message);
+    if (context->capabilities & IBUS_CAP_FOCUS) {
+        bus_input_context_focus_in (context);
+        reply = ibus_message_new_method_return (message);
+    }
+    else {
+        reply = ibus_message_new_error (message,
+                                        DBUS_ERROR_FAILED,
+                                        "The input context does not support focus.");
+    }
 
     return reply;
 }
@@ -832,9 +826,15 @@ _ic_focus_out (BusInputContext  *context,
 
     IBusMessage *reply;
 
-    bus_input_context_focus_out (context);
-
-    reply = ibus_message_new_method_return (message);
+    if (context->capabilities & IBUS_CAP_FOCUS) {
+        bus_input_context_focus_out (context);
+        reply = ibus_message_new_method_return (message);
+    }
+    else {
+        reply = ibus_message_new_error (message,
+                                        DBUS_ERROR_FAILED,
+                                        "The input context does not support focus.");
+    }
 
     return reply;
 }
@@ -850,7 +850,7 @@ _ic_reset (BusInputContext  *context,
 
     IBusMessage *reply;
 
-    if (context->engine) {
+    if (context->enabled && context->engine) {
         bus_engine_proxy_reset (context->engine);
     }
 
@@ -888,6 +888,12 @@ _ic_set_capabilities (BusInputContext  *context,
     if (context->capabilities != caps) {
         context->capabilities = caps;
 
+        /* If the context does not support IBUS_CAP_FOCUS, then we always assume
+         * it has focus. */
+        if ((caps & IBUS_CAP_FOCUS) == 0) {
+            bus_input_context_focus_in (context);
+        }
+
         if (context->engine) {
             bus_engine_proxy_set_capabilities (context->engine, caps);
         }
@@ -897,6 +903,43 @@ _ic_set_capabilities (BusInputContext  *context,
     return reply;
 }
 
+static IBusMessage *
+_ic_property_activate (BusInputContext  *context,
+                       IBusMessage      *message,
+                       BusConnection    *connection)
+{
+    g_assert (BUS_IS_INPUT_CONTEXT (context));
+    g_assert (message != NULL);
+    g_assert (BUS_IS_CONNECTION (connection));
+
+    IBusMessage *reply;
+    gchar *prop_name;
+    gint prop_state;
+    gboolean retval;
+    IBusError *error;
+
+    retval = ibus_message_get_args (message,
+                                    &error,
+                                    G_TYPE_STRING, &prop_name,
+                                    G_TYPE_INT, &prop_state,
+                                    G_TYPE_INVALID);
+
+    if (!retval) {
+        reply = ibus_message_new_error (message,
+                                        error->name,
+                                        error->message);
+        ibus_error_free (error);
+        return reply;
+    }
+
+    if (context->enabled && context->engine) {
+        bus_engine_proxy_property_activate (context->engine, prop_name, prop_state);
+    }
+
+    reply = ibus_message_new_method_return (message);
+    return reply;
+}
+
 static IBusMessage *
 _ic_enable (BusInputContext *context,
             IBusMessage     *message,
@@ -1069,20 +1112,19 @@ bus_input_context_ibus_message (BusInputContext *context,
         { IBUS_INTERFACE_INPUT_CONTEXT, "FocusOut",          _ic_focus_out },
         { IBUS_INTERFACE_INPUT_CONTEXT, "Reset",             _ic_reset },
         { IBUS_INTERFACE_INPUT_CONTEXT, "SetCapabilities",   _ic_set_capabilities },
+        { IBUS_INTERFACE_INPUT_CONTEXT, "PropertyActivate",  _ic_property_activate },
         { IBUS_INTERFACE_INPUT_CONTEXT, "Enable",            _ic_enable },
         { IBUS_INTERFACE_INPUT_CONTEXT, "Disable",           _ic_disable },
         { IBUS_INTERFACE_INPUT_CONTEXT, "IsEnabled",         _ic_is_enabled },
         { IBUS_INTERFACE_INPUT_CONTEXT, "SetEngine",         _ic_set_engine },
         { IBUS_INTERFACE_INPUT_CONTEXT, "GetEngine",         _ic_get_engine },
         { IBUS_INTERFACE_INPUT_CONTEXT, "Destroy",           _ic_destroy },
-
-        { NULL, NULL, NULL }
     };
 
     ibus_message_set_sender (message, bus_connection_get_unique_name (connection));
     ibus_message_set_destination (message, DBUS_SERVICE_DBUS);
 
-    for (i = 0; handlers[i].interface != NULL; i++) {
+    for (i = 0; i < G_N_ELEMENTS (handlers); i++) {
         if (ibus_message_is_method_call (message,
                                          handlers[i].interface,
                                          handlers[i].name)) {
@@ -1105,9 +1147,10 @@ bus_input_context_ibus_message (BusInputContext *context,
         }
     }
 
-    return parent_class->ibus_message ((IBusService *)context,
-                                       (IBusConnection *)connection,
-                                       message);
+    return IBUS_SERVICE_CLASS (bus_input_context_parent_class)->ibus_message (
+                                (IBusService *)context,
+                                (IBusConnection *)connection,
+                                message);
 }
 
 
@@ -1129,14 +1172,26 @@ bus_input_context_focus_in (BusInputContext *context)
 
     context->has_focus = TRUE;
 
+    // To make sure that we won't use an old value left before we losing focus
+    // last time.
+    context->prev_keyval = IBUS_VoidSymbol;
+    context->prev_modifiers = 0;
+
+    if (context->engine == NULL && context->enabled) {
+        g_signal_emit (context, context_signals[REQUEST_ENGINE], 0, NULL);
+    }
+
     if (context->engine && context->enabled) {
         bus_engine_proxy_focus_in (context->engine);
+        bus_engine_proxy_enable (context->engine);
+        bus_engine_proxy_set_capabilities (context->engine, context->capabilities);
+        bus_engine_proxy_set_cursor_location (context->engine, context->x, context->y, context->w, context->h);
     }
 
     if (context->capabilities & IBUS_CAP_FOCUS) {
         g_signal_emit (context, context_signals[FOCUS_IN], 0);
         if (context->engine && context->enabled) {
-            if (context->preedit_visible && (context->capabilities & IBUS_CAP_PREEDIT_TEXT) == 0) {
+          if (context->preedit_visible && !PREEDIT_CONDITION) {
                 g_signal_emit (context,
                                context_signals[UPDATE_PREEDIT_TEXT],
                                0,
@@ -1162,6 +1217,21 @@ bus_input_context_focus_in (BusInputContext *context)
     }
 }
 
+static void
+bus_input_context_clear_preedit_text (BusInputContext *context)
+{
+    g_assert (BUS_IS_INPUT_CONTEXT (context));
+
+    if (context->preedit_visible &&
+        context->preedit_mode == IBUS_ENGINE_PREEDIT_COMMIT) {
+        bus_input_context_commit_text (context, context->preedit_text);
+    }
+
+    /* always clear preedit text */
+    bus_input_context_update_preedit_text (context,
+        text_empty, 0, FALSE, IBUS_ENGINE_PREEDIT_CLEAR);
+}
+
 void
 bus_input_context_focus_out (BusInputContext *context)
 {
@@ -1170,36 +1240,31 @@ bus_input_context_focus_out (BusInputContext *context)
     if (!context->has_focus)
         return;
 
-    context->has_focus = FALSE;
+    bus_input_context_clear_preedit_text (context);
+    bus_input_context_update_auxiliary_text (context, text_empty, FALSE);
+    bus_input_context_update_lookup_table (context, lookup_table_empty, FALSE);
+    bus_input_context_register_properties (context, props_empty);
 
     if (context->engine && context->enabled) {
         bus_engine_proxy_focus_out (context->engine);
     }
 
+    context->has_focus = FALSE;
+
     if (context->capabilities & IBUS_CAP_FOCUS) {
-        if (context->preedit_visible && (context->capabilities & IBUS_CAP_PREEDIT_TEXT) == 0) {
-            g_signal_emit (context, context_signals[HIDE_PREEDIT_TEXT], 0);
-        }
-        if (context->auxiliary_visible && (context->capabilities & IBUS_CAP_AUXILIARY_TEXT) == 0) {
-            g_signal_emit (context, context_signals[HIDE_AUXILIARY_TEXT], 0);
-        }
-        if (context->auxiliary_visible && (context->capabilities & IBUS_CAP_LOOKUP_TABLE) == 0) {
-            g_signal_emit (context, context_signals[HIDE_LOOKUP_TABLE], 0);
-        }
         g_signal_emit (context, context_signals[FOCUS_OUT], 0);
     }
 }
 
-#define DEFINE_FUNC(name)                                   \
-    void                                                    \
-    bus_input_context_##name (BusInputContext *context)     \
-    {                                                       \
-        g_assert (BUS_IS_INPUT_CONTEXT (context));          \
-                                                            \
-                                                            \
-        if (context->engine) {                              \
-            bus_engine_proxy_##name (context->engine);      \
-        }                                                   \
+#define DEFINE_FUNC(name)                                                   \
+    void                                                                    \
+    bus_input_context_##name (BusInputContext *context)                     \
+    {                                                                       \
+        g_assert (BUS_IS_INPUT_CONTEXT (context));                          \
+                                                                            \
+        if (context->has_focus && context->enabled && context->engine) {    \
+            bus_engine_proxy_##name (context->engine);                      \
+        }                                                                   \
     }
 
 DEFINE_FUNC(page_up)
@@ -1217,7 +1282,7 @@ bus_input_context_candidate_clicked (BusInputContext *context,
 {
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
-    if (context->engine) {
+    if (context->enabled && context->engine) {
         bus_engine_proxy_candidate_clicked (context->engine,
                                             index,
                                             button,
@@ -1232,16 +1297,35 @@ bus_input_context_property_activate (BusInputContext *context,
 {
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
-    if (context->engine) {
+    if (context->enabled && context->engine) {
         bus_engine_proxy_property_activate (context->engine, prop_name, prop_state);
     }
 }
 
+static void
+bus_input_context_commit_text (BusInputContext *context,
+                               IBusText        *text)
+{
+    g_assert (BUS_IS_INPUT_CONTEXT (context));
+
+    if (!context->enabled)
+        return;
+
+    if (text == text_empty || text == NULL)
+        return;
+
+    bus_input_context_send_signal (context,
+                                   "CommitText",
+                                   IBUS_TYPE_TEXT, &text,
+                                   G_TYPE_INVALID);
+}
+
 static void
 bus_input_context_update_preedit_text (BusInputContext *context,
                                        IBusText        *text,
                                        guint            cursor_pos,
-                                       gboolean         visible)
+                                       gboolean         visible,
+                                       guint            mode)
 {
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
@@ -1249,11 +1333,12 @@ bus_input_context_update_preedit_text (BusInputContext *context,
         g_object_unref (context->preedit_text);
     }
 
-    context->preedit_text = (IBusText *) g_object_ref (text ? text : text_empty);
+    context->preedit_text = (IBusText *) g_object_ref_sink (text ? text : text_empty);
     context->preedit_cursor_pos = cursor_pos;
     context->preedit_visible = visible;
+    context->preedit_mode = mode;
 
-    if (context->capabilities & IBUS_CAP_PREEDIT_TEXT) {
+    if (PREEDIT_CONDITION) {
         bus_input_context_send_signal (context,
                                        "UpdatePreeditText",
                                        IBUS_TYPE_TEXT, &(context->preedit_text),
@@ -1282,7 +1367,7 @@ bus_input_context_show_preedit_text (BusInputContext *context)
 
     context->preedit_visible = TRUE;
 
-    if ((context->capabilities & IBUS_CAP_PREEDIT_TEXT) == IBUS_CAP_PREEDIT_TEXT) {
+    if (PREEDIT_CONDITION) {
         bus_input_context_send_signal (context,
                                        "ShowPreeditText",
                                        G_TYPE_INVALID);
@@ -1305,7 +1390,7 @@ bus_input_context_hide_preedit_text (BusInputContext *context)
 
     context->preedit_visible = FALSE;
 
-    if ((context->capabilities & IBUS_CAP_PREEDIT_TEXT) == IBUS_CAP_PREEDIT_TEXT) {
+    if (PREEDIT_CONDITION) {
         bus_input_context_send_signal (context,
                                        "HidePreeditText",
                                        G_TYPE_INVALID);
@@ -1328,7 +1413,7 @@ bus_input_context_update_auxiliary_text (BusInputContext *context,
         g_object_unref (context->auxiliary_text);
     }
 
-    context->auxiliary_text = (IBusText *) g_object_ref (text ? text : text_empty);
+    context->auxiliary_text = (IBusText *) g_object_ref_sink (text ? text : text_empty);
     context->auxiliary_visible = visible;
 
     if (context->capabilities & IBUS_CAP_AUXILIARY_TEXT) {
@@ -1404,7 +1489,7 @@ bus_input_context_update_lookup_table (BusInputContext *context,
         g_object_unref (context->lookup_table);
     }
 
-    context->lookup_table = (IBusLookupTable *) g_object_ref (table ? table : lookup_table_empty);
+    context->lookup_table = (IBusLookupTable *) g_object_ref_sink (table ? table : lookup_table_empty);
     context->lookup_table_visible = visible;
 
     if (context->capabilities & IBUS_CAP_LOOKUP_TABLE) {
@@ -1604,8 +1689,7 @@ _engine_destroy_cb (BusEngineProxy  *engine,
 
     g_assert (context->engine == engine);
 
-    bus_input_context_unset_engine (context);
-    bus_input_context_disable (context);
+    bus_input_context_set_engine (context, NULL);
 }
 
 static void
@@ -1619,11 +1703,7 @@ _engine_commit_text_cb (BusEngineProxy  *engine,
 
     g_assert (context->engine == engine);
 
-    bus_input_context_send_signal (context,
-                                   "CommitText",
-                                   IBUS_TYPE_TEXT, &text,
-                                   G_TYPE_INVALID);
-
+    bus_input_context_commit_text (context, text);
 }
 
 static void
@@ -1638,13 +1718,15 @@ _engine_forward_key_event_cb (BusEngineProxy    *engine,
 
     g_assert (context->engine == engine);
 
+    if (!context->enabled)
+        return;
+
     bus_input_context_send_signal (context,
                                    "ForwardKeyEvent",
                                    G_TYPE_UINT,  &keyval,
                                    G_TYPE_UINT,  &keycode,
                                    G_TYPE_UINT,  &state,
                                    G_TYPE_INVALID);
-
 }
 
 static void
@@ -1658,12 +1740,14 @@ _engine_delete_surrounding_text_cb (BusEngineProxy    *engine,
 
     g_assert (context->engine == engine);
 
+    if (!context->enabled)
+        return;
+
     bus_input_context_send_signal (context,
                                    "DeleteSurroundingText",
                                    G_TYPE_INT,   &offset_from_cursor,
                                    G_TYPE_UINT,  &nchars,
                                    G_TYPE_INVALID);
-
 }
 
 static void
@@ -1671,6 +1755,7 @@ _engine_update_preedit_text_cb (BusEngineProxy  *engine,
                                 IBusText        *text,
                                 guint            cursor_pos,
                                 gboolean         visible,
+                                guint            mode,
                                 BusInputContext *context)
 {
     g_assert (BUS_IS_ENGINE_PROXY (engine));
@@ -1679,7 +1764,10 @@ _engine_update_preedit_text_cb (BusEngineProxy  *engine,
 
     g_assert (context->engine == engine);
 
-    bus_input_context_update_preedit_text (context, text, cursor_pos, visible);
+    if (!context->enabled)
+        return;
+
+    bus_input_context_update_preedit_text (context, text, cursor_pos, visible, mode);
 }
 
 static void
@@ -1694,6 +1782,9 @@ _engine_update_auxiliary_text_cb (BusEngineProxy   *engine,
 
     g_assert (context->engine == engine);
 
+    if (!context->enabled)
+        return;
+
     bus_input_context_update_auxiliary_text (context, text, visible);
 }
 
@@ -1709,6 +1800,9 @@ _engine_update_lookup_table_cb (BusEngineProxy   *engine,
 
     g_assert (context->engine == engine);
 
+    if (!context->enabled)
+        return;
+
     bus_input_context_update_lookup_table (context, table, visible);
 }
 
@@ -1723,6 +1817,9 @@ _engine_register_properties_cb (BusEngineProxy  *engine,
 
     g_assert (context->engine == engine);
 
+    if (!context->enabled)
+        return;
+
     bus_input_context_register_properties (context, props);
 }
 
@@ -1737,6 +1834,9 @@ _engine_update_property_cb (BusEngineProxy  *engine,
 
     g_assert (context->engine == engine);
 
+    if (!context->enabled)
+        return;
+
     bus_input_context_update_property (context, prop);
 }
 
@@ -1750,8 +1850,11 @@ _engine_update_property_cb (BusEngineProxy  *engine,
                                                                 \
         g_assert (context->engine == engine);                   \
                                                                 \
+        if (!context->enabled)                                  \
+            return;                                             \
+                                                                \
         bus_input_context_##name (context);                     \
-}
+    }
 
 DEFINE_FUNCTION (show_preedit_text)
 DEFINE_FUNCTION (hide_preedit_text)
@@ -1770,8 +1873,14 @@ bus_input_context_enable (BusInputContext *context)
 {
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
+    if (!context->has_focus) {
+        context->enabled = TRUE;
+        /* TODO: Do we need to emit "enabled" signal? */
+        return;
+    }
+
     if (context->engine == NULL) {
-            g_signal_emit (context, context_signals[REQUEST_ENGINE], 0, NULL);
+        g_signal_emit (context, context_signals[REQUEST_ENGINE], 0, NULL);
     }
 
     if (context->engine == NULL)
@@ -1780,15 +1889,16 @@ bus_input_context_enable (BusInputContext *context)
     context->enabled = TRUE;
 
     bus_engine_proxy_enable (context->engine);
+    bus_engine_proxy_focus_in (context->engine);
+    bus_engine_proxy_set_capabilities (context->engine, context->capabilities);
+    bus_engine_proxy_set_cursor_location (context->engine, context->x, context->y, context->w, context->h);
+
     bus_input_context_send_signal (context,
                                    "Enabled",
                                    G_TYPE_INVALID);
     g_signal_emit (context,
                    context_signals[ENABLED],
                    0);
-    if (context->has_focus) {
-        bus_engine_proxy_focus_in (context->engine);
-    }
 }
 
 void
@@ -1796,10 +1906,13 @@ bus_input_context_disable (BusInputContext *context)
 {
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
+    bus_input_context_clear_preedit_text (context);
+    bus_input_context_update_auxiliary_text (context, text_empty, FALSE);
+    bus_input_context_update_lookup_table (context, lookup_table_empty, FALSE);
+    bus_input_context_register_properties (context, props_empty);
+
     if (context->engine) {
-        if (context->has_focus) {
-            bus_engine_proxy_focus_out (context->engine);
-        }
+        bus_engine_proxy_focus_out (context->engine);
         bus_engine_proxy_disable (context->engine);
     }
 
@@ -1852,22 +1965,20 @@ bus_input_context_unset_engine (BusInputContext *context)
 {
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
-    bus_input_context_register_properties (context, props_empty);
-    bus_input_context_update_preedit_text (context, text_empty, 0, FALSE);
+    bus_input_context_clear_preedit_text (context);
     bus_input_context_update_auxiliary_text (context, text_empty, FALSE);
     bus_input_context_update_lookup_table (context, lookup_table_empty, FALSE);
+    bus_input_context_register_properties (context, props_empty);
 
     if (context->engine) {
         gint i;
         for (i = 0; signals[i].name != NULL; i++) {
             g_signal_handlers_disconnect_by_func (context->engine, signals[i].callback, context);
         }
-        /* we destroy the engine */
-        ibus_object_destroy ((IBusObject *) context->engine);
+        /* Do not destroy the engine anymore, because of global engine feature */
         g_object_unref (context->engine);
         context->engine = NULL;
     }
-
 }
 
 void
@@ -1876,6 +1987,9 @@ bus_input_context_set_engine (BusInputContext *context,
 {
     g_assert (BUS_IS_INPUT_CONTEXT (context));
 
+    if (context->engine == engine)
+        return;
+
     if (context->engine != NULL) {
         bus_input_context_unset_engine (context);
     }
@@ -1886,7 +2000,7 @@ bus_input_context_set_engine (BusInputContext *context,
     else {
         gint i;
         context->engine = engine;
-        g_object_ref (context->engine);
+        g_object_ref_sink (context->engine);
 
         for (i = 0; signals[i].name != NULL; i++) {
             g_signal_connect (context->engine,
@@ -1894,16 +2008,12 @@ bus_input_context_set_engine (BusInputContext *context,
                               signals[i].callback,
                               context);
         }
-        bus_engine_proxy_set_cursor_location (context->engine, context->x, context->y, context->w, context->h);
-        if (context->enabled) {
+        if (context->has_focus && context->enabled) {
+            bus_engine_proxy_focus_in (context->engine);
             bus_engine_proxy_enable (context->engine);
-            if (context->has_focus) {
-                bus_engine_proxy_focus_in (context->engine);
-            }
+            bus_engine_proxy_set_capabilities (context->engine, context->capabilities);
+            bus_engine_proxy_set_cursor_location (context->engine, context->x, context->y, context->w, context->h);
         }
-
-        bus_engine_proxy_set_capabilities (context->engine, context->capabilities);
-
     }
     g_signal_emit (context,
                    context_signals[ENGINE_CHANGED],
@@ -1928,11 +2038,16 @@ bus_input_context_filter_keyboard_shortcuts (BusInputContext    *context,
 
     gboolean retval = FALSE;
 
-    static GQuark trigger;
-    static GQuark next_factory;
-    static GQuark prev_factory;
-
-    GQuark event;
+    if (context->filter_release){
+        if(modifiers & IBUS_RELEASE_MASK) {
+            /* filter release key event */
+            return TRUE;
+        }
+        else {
+            /* stop filter release key event */
+            context->filter_release = FALSE;
+        }
+    }
 
     if (keycode != 0 && !BUS_DEFAULT_IBUS->use_sys_layout) {
         IBusKeymap *keymap = BUS_DEFAULT_KEYMAP;
@@ -1945,65 +2060,17 @@ bus_input_context_filter_keyboard_shortcuts (BusInputContext    *context,
         }
     }
 
-    if (trigger == 0) {
-        trigger = g_quark_from_static_string ("trigger");
-        next_factory = g_quark_from_static_string ("next-engine");
-        prev_factory = g_quark_from_static_string ("prev-engine");
-    }
-
-    if (context->filter_release){
-        if(modifiers & IBUS_RELEASE_MASK) {
-            /* filter release key event */
-            return TRUE;
-        }
-        else {
-            /* stop filter release key event */
-            context->filter_release = FALSE;
-        }
-    }
-
-    event = ibus_hotkey_profile_filter_key_event (BUS_DEFAULT_HOTKEY_PROFILE,
-                                                  keyval,
-                                                  modifiers,
-                                                  context->prev_keyval,
-                                                  context->prev_modifiers,
-                                                  0);
+    retval = bus_ibus_impl_filter_keyboard_shortcuts (BUS_DEFAULT_IBUS,
+                                                      context,
+                                                      keyval,
+                                                      modifiers,
+                                                      context->prev_keyval,
+                                                      context->prev_modifiers);
     context->prev_keyval = keyval;
     context->prev_modifiers = modifiers;
 
-    if (event == trigger) {
-        gboolean enabled = context->enabled;
-
-        if (context->enabled) {
-            bus_input_context_disable (context);
-        }
-        else {
-            bus_input_context_enable (context);
-        }
-
-        retval = (enabled != context->enabled);
-    }
-    else if (event == next_factory) {
-        if (context->engine == NULL || context->enabled == FALSE) {
-            retval =  FALSE;
-        }
-        else {
-            g_signal_emit (context, context_signals[REQUEST_NEXT_ENGINE], 0);
-            retval = TRUE;
-        }
-    }
-    else if (event == prev_factory) {
-        if (context->engine == NULL || context->enabled == FALSE) {
-            retval = FALSE;
-        }
-        else {
-            g_signal_emit (context, context_signals[REQUEST_PREV_ENGINE], 0);
-            retval = TRUE;
-        }
-    }
-
     if (retval == TRUE) {
-        /* begine filter release key event */
+        /* begin filter release key event */
         context->filter_release = TRUE;
     }
 
@@ -2040,3 +2107,10 @@ bus_input_context_send_signal (BusInputContext *context,
 
     return retval;
 }
+
+guint
+bus_input_context_get_capabilities (BusInputContext *context)
+{
+    g_assert (BUS_IS_INPUT_CONTEXT (context));
+    return context->capabilities;
+}
