@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* IBus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,52 +19,24 @@
  * Boston, MA 02111-1307, USA.
  */
 #include <string.h>
+#include <dbus/dbus.h>
 #include "matchrule.h"
 
 #define BUS_CONFIG_PROXY_GET_PRIVATE(o)  \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), BUS_TYPE_CONFIG_PROXY, BusMatchRulePrivate))
 
 
-static void      bus_match_rule_class_init      (BusMatchRuleClass  *klass);
-static void      bus_match_rule_init            (BusMatchRule       *rule);
 static void      bus_match_rule_destroy         (BusMatchRule       *rule);
 static void      _connection_destroy_cb         (BusConnection      *connection,
                                                  BusMatchRule       *rule);
-static IBusObjectClass  *parent_class = NULL;
 
-GType
-bus_match_rule_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (BusMatchRuleClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    bus_match_rule_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (BusMatchRule),
-        0,
-        (GInstanceInitFunc) bus_match_rule_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_OBJECT,
-                    "BusMatchRule",
-                    &type_info,
-                    (GTypeFlags)0);
-    }
-    return type;
-}
+G_DEFINE_TYPE (BusMatchRule, bus_match_rule, IBUS_TYPE_OBJECT)
 
 static void
 bus_match_rule_class_init (BusMatchRuleClass *klass)
 {
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (klass);
 
-    parent_class = (IBusObjectClass *) g_type_class_peek_parent (klass);
-
     ibus_object_class->destroy = (IBusObjectDestroyFunc) bus_match_rule_destroy;
 }
 
@@ -107,7 +79,7 @@ bus_match_rule_destroy (BusMatchRule *rule)
     }
     g_list_free (rule->recipients);
 
-    IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (rule));
+    IBUS_OBJECT_CLASS(bus_match_rule_parent_class)->destroy (IBUS_OBJECT (rule));
 }
 
 
@@ -607,7 +579,7 @@ bus_match_rule_add_recipient (BusMatchRule   *rule,
 
     recipient = g_slice_new (BusRecipient);
 
-    g_object_ref (connection);
+    g_object_ref_sink (connection);
     recipient->connection = connection;
     recipient->refcount  = 1;
 
@@ -667,7 +639,6 @@ bus_match_rule_get_recipients (BusMatchRule   *rule,
     for (link = rule->recipients; link != NULL; link = link->next) {
         BusRecipient *recipient = (BusRecipient *) link->data;
 
-        g_object_ref (recipient->connection);
         recipients = g_list_append (recipients, recipient->connection);
     }
 
