@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* bus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,7 @@
 
 G_BEGIN_DECLS
 
-extern gboolean g_rescan;
+extern gchar *g_cache;
 extern gboolean g_mempro;
 extern gboolean g_verbose;
 extern gint   g_dbus_timeout;
