@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+#include <dbus/dbus.h>
 #include <ibusinternal.h>
 #include <ibusmarshalers.h>
 #include "panelproxy.h"
@@ -39,7 +39,6 @@ static guint    panel_signals[LAST_SIGNAL] = { 0 };
 // static guint            engine_signals[LAST_SIGNAL] = { 0 };
 
 /* functions prototype */
-static void     bus_panel_proxy_class_init      (BusPanelProxyClass     *klass);
 static void     bus_panel_proxy_init            (BusPanelProxy          *panel);
 static void     bus_panel_proxy_real_destroy    (BusPanelProxy          *panel);
 
@@ -59,40 +58,8 @@ static void     bus_panel_proxy_property_activate
                                                  const gchar            *prop_name,
                                                  gint                    prop_state);
 
-static IBusProxyClass  *parent_class = NULL;
 
-struct _SignalCallbackTable {
-    gchar *name;
-    GCallback callback;
-} ;
-
-static const struct _SignalCallbackTable __signals[];
-
-GType
-bus_panel_proxy_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (BusPanelProxyClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    bus_panel_proxy_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (BusPanelProxy),
-        0,
-        (GInstanceInitFunc) bus_panel_proxy_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_PROXY,
-                    "BusPanelProxy",
-                    &type_info,
-                    (GTypeFlags)0);
-    }
-    return type;
-}
+G_DEFINE_TYPE(BusPanelProxy, bus_panel_proxy, IBUS_TYPE_PROXY)
 
 BusPanelProxy *
 bus_panel_proxy_new (BusConnection *connection)
@@ -115,9 +82,6 @@ bus_panel_proxy_class_init (BusPanelProxyClass *klass)
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (klass);
     IBusProxyClass *proxy_class = IBUS_PROXY_CLASS (klass);
 
-
-    parent_class = (IBusProxyClass *) g_type_class_peek_parent (klass);
-
     klass->page_up     = bus_panel_proxy_page_up;
     klass->page_down   = bus_panel_proxy_page_down;
     klass->cursor_up   = bus_panel_proxy_cursor_up;
@@ -232,7 +196,7 @@ bus_panel_proxy_real_destroy (BusPanelProxy *panel)
         panel->focused_context = NULL;
     }
 
-    IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (panel));
+    IBUS_OBJECT_CLASS(bus_panel_proxy_parent_class)->destroy (IBUS_OBJECT (panel));
 }
 
 static gboolean
@@ -254,14 +218,11 @@ bus_panel_proxy_ibus_signal (IBusProxy      *proxy,
         { "PageDown",       PAGE_DOWN },
         { "CursorUp",       CURSOR_UP },
         { "CursorDown",     CURSOR_DOWN },
-        { NULL, 0},
     };
 
     panel = BUS_PANEL_PROXY (proxy);
 
-    for (i = 0; ; i++) {
-        if (signals[i].member == NULL)
-            break;
+    for (i = 0; i < G_N_ELEMENTS (signals); i++) {
         if (ibus_message_is_signal (message, IBUS_INTERFACE_PANEL, signals[i].member)) {
             g_signal_emit (panel, panel_signals[signals[i].signal_id], 0);
             goto handled;
@@ -334,66 +295,6 @@ failed:
 }
 
 
-void
-bus_panel_proxy_focus_in (BusPanelProxy     *panel,
-                          BusInputContext   *context)
-{
-    g_assert (BUS_IS_PANEL_PROXY (panel));
-    g_assert (BUS_IS_INPUT_CONTEXT (context));
-
-    if (panel->focused_context == context)
-        return;
-
-    if (panel->focused_context != NULL)
-        bus_panel_proxy_focus_out (panel, panel->focused_context);
-
-    g_object_ref (context);
-    panel->focused_context = context;
-
-    const gchar *path = ibus_service_get_path ((IBusService *)context);
-
-    ibus_proxy_call ((IBusProxy *) panel,
-                     "FocusIn",
-                     IBUS_TYPE_OBJECT_PATH, &path,
-                     G_TYPE_INVALID);
-
-    /* install signal handlers */
-    gint i;
-    for (i = 0; __signals[i].name != NULL; i++) {
-        g_signal_connect (context,
-                          __signals[i].name,
-                          __signals[i].callback,
-                          panel);
-    }
-}
-
-void
-bus_panel_proxy_focus_out (BusPanelProxy    *panel,
-                           BusInputContext  *context)
-{
-    g_assert (BUS_IS_PANEL_PROXY (panel));
-    g_assert (BUS_IS_INPUT_CONTEXT (context));
-
-    g_assert (panel->focused_context == context);
-
-    /* uninstall signal handlers */
-    gint i;
-    for (i = 0; __signals[i].name != NULL; i++) {
-        g_signal_handlers_disconnect_by_func (context,
-                                              __signals[i].callback,
-                                              panel);
-    }
-
-    const gchar *path = ibus_service_get_path ((IBusService *)context);
-
-    ibus_proxy_call ((IBusProxy *) panel,
-                     "FocusOut",
-                     IBUS_TYPE_OBJECT_PATH, &path,
-                     G_TYPE_INVALID);
-
-    g_object_unref (panel->focused_context);
-    panel->focused_context = NULL;
-}
 
 void
 bus_panel_proxy_set_cursor_location (BusPanelProxy *panel,
@@ -690,8 +591,10 @@ DEFINE_FUNCTION (state_changed)
 
 #undef DEFINE_FUNCTION
 
-static const struct _SignalCallbackTable
-__signals[] = {
+static const struct _SignalCallbackTable {
+    gchar *name;
+    GCallback callback;
+} __signals[] = {
     { "set-cursor-location",        G_CALLBACK (_context_set_cursor_location_cb) },
 
     { "update-preedit-text",        G_CALLBACK (_context_update_preedit_text_cb) },
@@ -715,9 +618,69 @@ __signals[] = {
 
     { "enabled",                    G_CALLBACK (_context_state_changed_cb) },
     { "disabled",                   G_CALLBACK (_context_state_changed_cb) },
-    { "factory-changed",            G_CALLBACK (_context_state_changed_cb) },
+    { "engine-changed",             G_CALLBACK (_context_state_changed_cb) },
 
     //    { "destroy",                    G_CALLBACK (_context_destroy_cb) },
-    { NULL, NULL}
 };
 
+void
+bus_panel_proxy_focus_in (BusPanelProxy     *panel,
+                          BusInputContext   *context)
+{
+    g_assert (BUS_IS_PANEL_PROXY (panel));
+    g_assert (BUS_IS_INPUT_CONTEXT (context));
+
+    if (panel->focused_context == context)
+        return;
+
+    if (panel->focused_context != NULL)
+        bus_panel_proxy_focus_out (panel, panel->focused_context);
+
+    g_object_ref_sink (context);
+    panel->focused_context = context;
+
+    const gchar *path = ibus_service_get_path ((IBusService *)context);
+
+    ibus_proxy_call ((IBusProxy *) panel,
+                     "FocusIn",
+                     IBUS_TYPE_OBJECT_PATH, &path,
+                     G_TYPE_INVALID);
+
+    /* install signal handlers */
+    gint i;
+    for (i = 0; i < G_N_ELEMENTS (__signals); i++) {
+        g_signal_connect (context,
+                          __signals[i].name,
+                          __signals[i].callback,
+                          panel);
+    }
+}
+
+void
+bus_panel_proxy_focus_out (BusPanelProxy    *panel,
+                           BusInputContext  *context)
+{
+    g_assert (BUS_IS_PANEL_PROXY (panel));
+    g_assert (BUS_IS_INPUT_CONTEXT (context));
+
+    g_assert (panel->focused_context == context);
+
+    /* uninstall signal handlers */
+    gint i;
+    for (i = 0; i < G_N_ELEMENTS (__signals); i++) {
+        g_signal_handlers_disconnect_by_func (context,
+                                              __signals[i].callback,
+                                              panel);
+    }
+
+    const gchar *path = ibus_service_get_path ((IBusService *)context);
+
+    ibus_proxy_call ((IBusProxy *) panel,
+                     "FocusOut",
+                     IBUS_TYPE_OBJECT_PATH, &path,
+                     G_TYPE_INVALID);
+
+    g_object_unref (panel->focused_context);
+    panel->focused_context = NULL;
+}
+
