@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus
- * Copyright (C) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2007-2009 Red Hat, Inc.
+ * Copyright (C) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2007-2010 Red Hat, Inc.
  *
  * main.c:
  *
@@ -20,6 +20,8 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#define _GNU_SOURCE
+
 #include <X11/Xproto.h>
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
@@ -37,7 +39,6 @@
 #include <signal.h>
 #include <stdlib.h>
 
-#define _GNU_SOURCES
 #include <getopt.h>
 
 #define LOG(level, fmt_args...) \
@@ -106,8 +107,8 @@ static void     _context_disabled_cb        (IBusInputContext   *context,
 static GHashTable     *_x11_ic_table = NULL;
 static GHashTable     *_connections = NULL;
 static XIMS _xims = NULL;
-static gchar _server_name[128] = "ibus";
-static gchar _locale[1024] = LOCALES_STRING;
+static gchar *_server_name = NULL;
+static gchar *_locale = NULL;
 
 static gboolean _kill_daemon = FALSE;
 static gint     g_debug_level = 0;
@@ -446,6 +447,7 @@ xim_forward_event (XIMS xims, IMForwardEventStruct *call_data)
     X11IC *x11ic;
     XKeyEvent *xevent;
     GdkEventKey event;
+    gboolean retval;
 
     LOG (1, "XIM_FORWARD_EVENT ic=%d connect_id=%d",
                 call_data->icid, call_data->connect_id);
@@ -466,10 +468,11 @@ xim_forward_event (XIMS xims, IMForwardEventStruct *call_data)
         event.state |= IBUS_RELEASE_MASK;
     }
 
-    if (ibus_input_context_process_key_event (x11ic->context,
-                                              event.keyval,
-                                              event.hardware_keycode - 8,
-                                              event.state)) {
+    retval = ibus_input_context_process_key_event (x11ic->context,
+                                                   event.keyval,
+                                                   event.hardware_keycode - 8,
+                                                   event.state);
+    if (retval) {
         if (! x11ic->has_preedit_area) {
             _xim_set_cursor_location (x11ic);
         }
@@ -743,7 +746,7 @@ _xim_forward_key_event (X11IC   *x11ic,
 
     xkp.xkey.time = 0;
     xkp.xkey.state = state;
-    xkp.xkey.keycode = keycode;
+    xkp.xkey.keycode = (keycode == 0) ? 0 : keycode + 8;
 
     fe.major_code = XIM_FORWARD_EVENT;
     fe.icid = x11ic->icid;
@@ -897,14 +900,11 @@ _init_ibus (void)
 {
     if (_bus != NULL)
         return;
+
     ibus_init ();
 
     _bus = ibus_bus_new ();
 
-    if (!ibus_bus_is_connected (_bus)) {
-        g_error ("Can not connect to ibus-daemon!");
-    }
-
     g_signal_connect (_bus, "disconnected",
                         G_CALLBACK (_bus_disconnected_cb), NULL);
 }
@@ -963,8 +963,8 @@ _xim_init_IMdkit ()
     _xims = IMOpenIM(GDK_DISPLAY(),
         IMModifiers, "Xi18n",
         IMServerWindow, GDK_WINDOW_XWINDOW(win),
-        IMServerName, _server_name,
-        IMLocale, _locale,
+        IMServerName, _server_name != NULL ? _server_name : "ibus",
+        IMLocale, _locale != NULL ? _locale : LOCALES_STRING,
         IMServerTransport, "X/",
         IMInputStyles, &styles,
         IMEncodingList, &encodings,
@@ -976,7 +976,7 @@ _xim_init_IMdkit ()
 
     if (!ibus_bus_is_connected (_bus)) {
         g_warning ("Can not connect to ibus daemon");
-        exit (1);
+        exit (EXIT_FAILURE);
     }
 }
 
@@ -1049,13 +1049,18 @@ main (int argc, char **argv)
                 g_debug_level = atoi (optarg);
             }
             else if (g_strcmp0 (long_options[option_index].name, "server-name") == 0) {
-                strncpy (_server_name, optarg, sizeof (_server_name));
+                g_free (_server_name);
+                _server_name = g_strdup (optarg);
             }
             else if (g_strcmp0 (long_options[option_index].name, "locale") == 0) {
-                strncpy (_locale, optarg, sizeof (_locale));
+                g_free (_locale);
+                _locale = g_strdup (optarg);
             }
             else if (g_strcmp0 (long_options[option_index].name, "locale-append") == 0) {
-                strncat (_locale, optarg, sizeof (_locale));
+                gchar *tmp = g_strdup_printf ("%s,%s",
+                                _locale != NULL ? _locale : LOCALES_STRING, optarg);
+                g_free (_locale);
+                _locale = tmp;
             }
             else if (g_strcmp0 (long_options[option_index].name, "help") == 0) {
                 _print_usage (stdout, argv[0]);
@@ -1069,13 +1074,19 @@ main (int argc, char **argv)
             g_debug_level = atoi (optarg);
             break;
         case 'n':
-            strncpy (_server_name, optarg, sizeof (_server_name));
+            g_free (_server_name);
+            _server_name = g_strdup (optarg);
             break;
         case 'l':
-            strncpy (_locale, optarg, sizeof (_locale));
+            g_free (_locale);
+            _locale = g_strdup (optarg);
             break;
-        case 'a':
-            strncat (_locale, optarg, sizeof (_locale));
+        case 'a': {
+                gchar *tmp = g_strdup_printf ("%s,%s",
+                                _locale != NULL ? _locale : LOCALES_STRING, optarg);
+                g_free (_locale);
+                _locale = tmp;
+            }
             break;
         case 'k':
             _kill_daemon = TRUE;
