@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright(c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright(c) 2007-2009 Red Hat, Inc.
+# Copyright(c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright(c) 2007-2010 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -104,8 +104,8 @@ class IEngine(dbus.service.Object):
     @signal(signature="uu")
     def ForwardKeyEvent(self, keyval, state): pass
 
-    @signal(signature="vub")
-    def UpdatePreeditText(self, text, cursor_pos, visible): pass
+    @signal(signature="vubu")
+    def UpdatePreeditText(self, text, cursor_pos, visible, mode): pass
 
     @signal()
     def ShowPreeditText(self): pass
