@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <glib/gstdio.h>
 #include <gio/gio.h>
+#include <dbus/dbus.h>
 #include "ibusbus.h"
 #include "ibusinternal.h"
 #include "ibusshare.h"
@@ -37,6 +39,7 @@
 enum {
     CONNECTED,
     DISCONNECTED,
+    GLOBAL_ENGINE_CHANGED,
     LAST_SIGNAL,
 };
 
@@ -47,47 +50,26 @@ struct _IBusBusPrivate {
     IBusConnection *connection;
     gboolean watch_dbus_signal;
     IBusConfig *config;
+    gchar *unique_name;
 };
 typedef struct _IBusBusPrivate IBusBusPrivate;
 
 static guint    bus_signals[LAST_SIGNAL] = { 0 };
 
+static IBusBus *_bus = NULL;
+
 /* functions prototype */
-static void     ibus_bus_class_init     (IBusBusClass   *klass);
-static void     ibus_bus_init           (IBusBus        *bus);
+static GObject* ibus_bus_constructor    (GType          type,
+                                         guint          n_params,
+                                         GObjectConstructParam
+                                                        *params);
 static void     ibus_bus_destroy        (IBusObject     *object);
 static void     ibus_bus_watch_dbus_signal
                                         (IBusBus        *bus);
 static void     ibus_bus_unwatch_dbus_signal
                                         (IBusBus        *bus);
-static IBusObjectClass  *parent_class = NULL;
 
-GType
-ibus_bus_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (IBusBusClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_bus_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (IBusBus),
-        0,
-        (GInstanceInitFunc) ibus_bus_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_OBJECT,
-                    "IBusBus",
-                    &type_info,
-                    (GTypeFlags)0);
-    }
-
-    return type;
-}
+G_DEFINE_TYPE (IBusBus, ibus_bus, IBUS_TYPE_OBJECT)
 
 IBusBus *
 ibus_bus_new (void)
@@ -100,12 +82,12 @@ ibus_bus_new (void)
 static void
 ibus_bus_class_init (IBusBusClass *klass)
 {
+    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (klass);
 
-    parent_class = (IBusObjectClass *) g_type_class_peek_parent (klass);
-
     g_type_class_add_private (klass, sizeof (IBusBusPrivate));
 
+    gobject_class->constructor = ibus_bus_constructor;
     ibus_object_class->destroy = ibus_bus_destroy;
 
     // install signals
@@ -142,6 +124,39 @@ ibus_bus_class_init (IBusBusClass *klass)
             ibus_marshal_VOID__VOID,
             G_TYPE_NONE,
             0);
+
+    /**
+     * IBusBus::global-engine-changed:
+     *
+     * Emitted when global engine is changed.
+     *
+     * <note><para>Argument @user_data is ignored in this function.</para></note>
+     */
+    bus_signals[GLOBAL_ENGINE_CHANGED] =
+        g_signal_new (I_("global-engine-changed"),
+            G_TYPE_FROM_CLASS (klass),
+            G_SIGNAL_RUN_LAST,
+            0,
+            NULL, NULL,
+            ibus_marshal_VOID__VOID,
+            G_TYPE_NONE,
+            0);
+}
+
+static gboolean
+_connection_ibus_signal_cb (IBusConnection *connection,
+                            IBusMessage    *message,
+                            IBusBus        *bus)
+{
+    IBusBusPrivate *priv;
+    priv = IBUS_BUS_GET_PRIVATE (bus);
+
+    if (ibus_message_is_signal (message, IBUS_INTERFACE_IBUS,
+                                "GlobalEngineChanged")) {
+        g_signal_emit (bus, bus_signals[GLOBAL_ENGINE_CHANGED], 0);
+        return TRUE;
+    }
+    return FALSE;
 }
 
 static void
@@ -158,9 +173,15 @@ _connection_destroy_cb (IBusConnection  *connection,
     g_signal_handlers_disconnect_by_func (priv->connection,
                                           G_CALLBACK (_connection_destroy_cb),
                                           bus);
+    g_signal_handlers_disconnect_by_func (priv->connection,
+                                          G_CALLBACK (_connection_ibus_signal_cb),
+                                          bus);
     g_object_unref (priv->connection);
     priv->connection = NULL;
 
+    g_free (priv->unique_name);
+    priv->unique_name = NULL;
+
     g_signal_emit (bus, bus_signals[DISCONNECTED], 0);
 }
 
@@ -192,6 +213,7 @@ ibus_bus_connect (IBusBus *bus)
 
     if (priv->connection) {
         ibus_bus_hello (bus);
+
         g_signal_connect (priv->connection,
                           "destroy",
                           (GCallback) _connection_destroy_cb,
@@ -201,6 +223,18 @@ ibus_bus_connect (IBusBus *bus)
         if (priv->watch_dbus_signal) {
             ibus_bus_watch_dbus_signal (bus);
         }
+
+        /** Watch ibus signals. */
+        const gchar *rule =
+            "type='signal',"
+            "path='" IBUS_PATH_IBUS "',"
+            "interface='" IBUS_INTERFACE_IBUS "'";
+
+        ibus_bus_add_match (bus, rule);
+        g_signal_connect (priv->connection,
+                          "ibus-signal",
+                          (GCallback) _connection_ibus_signal_cb,
+                          bus);
     }
 }
 
@@ -218,7 +252,7 @@ _changed_cb (GFileMonitor       *monitor,
 
     if (ibus_bus_is_connected (bus))
         return;
-    
+
     ibus_bus_connect (bus);
 }
 
@@ -235,23 +269,19 @@ ibus_bus_init (IBusBus *bus)
     priv->config = NULL;
     priv->connection = NULL;
     priv->watch_dbus_signal = FALSE;
+    priv->unique_name = NULL;
 
     path = g_path_get_dirname (ibus_get_socket_path ());
 
+    g_mkdir_with_parents (path, 0700);
+    g_chmod (path, 0700);
+
     if (stat (path, &buf) == 0) {
-        if (buf.st_uid != ibus_get_daemon_uid ()) {
+        if (buf.st_uid != getuid ()) {
             g_warning ("The owner of %s is not %s!", path, ibus_get_user_name ());
             return;
         }
     }
-#if 0
-    else {
-        if (getuid () == ibus_get_daemon_uid ()) {
-            mkdir (path, 0700);
-            chmod (path, 0700);
-        }
-    }
-#endif
 
     ibus_bus_connect (bus);
 
@@ -265,6 +295,27 @@ ibus_bus_init (IBusBus *bus)
     g_free (path);
 }
 
+static GObject*
+ibus_bus_constructor (GType                  type,
+                      guint                  n_params,
+                      GObjectConstructParam *params)
+{
+    GObject *object;
+
+    /* share one IBusBus instance in whole application */
+    if (_bus == NULL) {
+        object = G_OBJECT_CLASS (ibus_bus_parent_class)->constructor (type, n_params, params);
+        /* make bus object sink */
+        g_object_ref_sink (object);
+        _bus = IBUS_BUS (object);
+    }
+    else {
+        object = g_object_ref (_bus);
+    }
+
+    return object;
+}
+
 static void
 ibus_bus_destroy (IBusObject *object)
 {
@@ -274,6 +325,9 @@ ibus_bus_destroy (IBusObject *object)
     bus = IBUS_BUS (object);
     priv = IBUS_BUS_GET_PRIVATE (bus);
 
+    g_assert (_bus == (IBusBus *)object);
+    _bus = NULL;
+
     if (priv->monitor) {
         g_object_unref (priv->monitor);
         priv->monitor = NULL;
@@ -289,7 +343,10 @@ ibus_bus_destroy (IBusObject *object)
         priv->connection = NULL;
     }
 
-    IBUS_OBJECT_CLASS (parent_class)->destroy (object);
+    g_free (priv->unique_name);
+    priv->unique_name = NULL;
+
+    IBUS_OBJECT_CLASS (ibus_bus_parent_class)->destroy (object);
 }
 
 gboolean
@@ -369,14 +426,14 @@ ibus_bus_create_input_context (IBusBus      *bus,
     return context;
 }
 
-static gboolean
-ibus_bus_call (IBusBus      *bus,
-               const gchar  *name,
-               const gchar  *path,
-               const gchar  *interface,
-               const gchar  *member,
-               GType         first_arg_type,
-               ...)
+IBusMessage *
+ibus_bus_call_with_reply_valist (IBusBus      *bus,
+                                 const gchar  *name,
+                                 const gchar  *path,
+                                 const gchar  *interface,
+                                 const gchar  *member,
+                                 GType         first_arg_type,
+                                 va_list       va_args)
 {
     g_assert (IBUS_IS_BUS (bus));
     g_assert (name != NULL);
@@ -386,9 +443,6 @@ ibus_bus_call (IBusBus      *bus,
 
     IBusMessage *message, *reply;
     IBusError *error;
-    va_list args;
-    GType type;
-    gboolean retval;
     IBusBusPrivate *priv;
 
     g_return_val_if_fail (ibus_bus_is_connected (bus), FALSE);
@@ -397,9 +451,7 @@ ibus_bus_call (IBusBus      *bus,
 
     message = ibus_message_new_method_call (name, path, interface, member);
 
-    va_start (args, first_arg_type);
-    ibus_message_append_args_valist (message, first_arg_type, args);
-    va_end (args);
+    ibus_message_append_args_valist (message, first_arg_type, va_args);
 
     reply = ibus_connection_send_with_reply_and_block (
                                         priv->connection,
@@ -411,66 +463,95 @@ ibus_bus_call (IBusBus      *bus,
     if (reply == NULL) {
         g_warning ("%s : %s", error->name, error->message);
         ibus_error_free (error);
-        return FALSE;
+        return NULL;
     }
 
     if ((error = ibus_error_new_from_message (reply)) != NULL) {
         g_warning ("%s : %s", error->name, error->message);
         ibus_error_free (error);
         ibus_message_unref (reply);
-        return FALSE;
+        return NULL;
     }
 
-    va_start (args, first_arg_type);
-
-    type = first_arg_type;
-
-    while (type != G_TYPE_INVALID) {
-        va_arg (args, gpointer);
-        type = va_arg (args, GType);
-    }
-
-    type = va_arg (args, GType);
-    if (type != G_TYPE_INVALID) {
-        retval = ibus_message_get_args_valist (reply, &error, type, args);
-    }
-    else {
-        retval = TRUE;
-    }
-    va_end (args);
-
-    ibus_message_unref (reply);
-
-    if (!retval) {
-        g_warning ("%s: %s", error->name, error->message);
-        ibus_error_free (error);
-        return FALSE;
-    }
-
-    return TRUE;
+    return reply;
 }
 
-const gchar *
+IBusMessage *
+ibus_bus_call_with_reply (IBusBus      *bus,
+                          const gchar  *name,
+                          const gchar  *path,
+                          const gchar  *interface,
+                          const gchar  *member,
+                          GType         first_arg_type,
+                          ...)
+{
+    IBusMessage *reply;
+    va_list va_args;
+
+    va_start (va_args, first_arg_type);
+    reply = ibus_bus_call_with_reply_valist (
+        bus, name, path, interface, member, first_arg_type, va_args);
+    va_end (va_args);
+
+    return reply;
+}
+
+gboolean
+ibus_bus_call (IBusBus      *bus,
+               const gchar  *name,
+               const gchar  *path,
+               const gchar  *interface,
+               const gchar  *member,
+               GType         first_arg_type,
+               ...)
+{
+    IBusMessage *reply;
+    va_list va_args;
+
+    va_start (va_args, first_arg_type);
+    reply = ibus_bus_call_with_reply_valist (
+        bus, name, path, interface, member, first_arg_type, va_args);
+    va_end (va_args);
+
+    if (reply) {
+      ibus_message_unref (reply);
+      return TRUE;
+    }
+
+    return FALSE;
+}
+
+gchar *
 ibus_bus_current_input_context(IBusBus      *bus)
 {
     g_assert (IBUS_IS_BUS (bus));
+    g_return_val_if_fail (ibus_bus_is_connected (bus), NULL);
 
-    gchar *name = NULL;
-    gboolean result;
+    gchar *path = NULL;
+    IBusMessage *reply = NULL;
+    IBusError *error = NULL;
 
-    result = ibus_bus_call (bus,
-                            DBUS_SERVICE_DBUS,
-                            DBUS_PATH_DBUS,
-                            DBUS_INTERFACE_DBUS,
-                            "CurrentInputContext",
-                            G_TYPE_INVALID,
-                            G_TYPE_STRING, &name,
-                            G_TYPE_INVALID);
+    reply = ibus_bus_call_with_reply (bus,
+                                      IBUS_SERVICE_IBUS,
+                                      IBUS_PATH_IBUS,
+                                      IBUS_INTERFACE_IBUS,
+                                      "CurrentInputContext",
+                                      G_TYPE_INVALID);
 
-    if (result)
-        return name;
+    if (reply) {
+        if (ibus_message_get_args (reply, &error,
+                                   IBUS_TYPE_OBJECT_PATH, &path,
+                                   G_TYPE_INVALID)) {
+            path = g_strdup (path);
+        } else {
+            g_warning ("%s: %s", error->name, error->message);
+            ibus_error_free (error);
+        }
 
-    return NULL;
+        ibus_message_unref (reply);
+    }
+
+    return path;
 }
 
 static void
@@ -532,21 +613,35 @@ ibus_bus_hello (IBusBus *bus)
     g_assert (IBUS_IS_BUS (bus));
 
     gchar *unique_name = NULL;
-    gboolean result;
+    IBusMessage *reply = NULL;
+    IBusError *error = NULL;
+    IBusBusPrivate *priv;
 
-    result = ibus_bus_call (bus,
-                            DBUS_SERVICE_DBUS,
-                            DBUS_PATH_DBUS,
-                            DBUS_INTERFACE_DBUS,
-                            "Hello",
-                            G_TYPE_INVALID,
-                            G_TYPE_STRING, &unique_name,
-                            G_TYPE_INVALID);
+    priv = IBUS_BUS_GET_PRIVATE (bus);
 
-    if (result)
-        return unique_name;
+    g_free (priv->unique_name);
+    priv->unique_name = NULL;
 
-    return NULL;
+    reply = ibus_bus_call_with_reply (bus,
+                                      DBUS_SERVICE_DBUS,
+                                      DBUS_PATH_DBUS,
+                                      DBUS_INTERFACE_DBUS,
+                                      "Hello",
+                                      G_TYPE_INVALID);
+
+    if (reply) {
+        if (ibus_message_get_args (reply, &error, G_TYPE_STRING, &unique_name,
+                                   G_TYPE_INVALID)) {
+            priv->unique_name = g_strdup (unique_name);
+        } else {
+            g_warning ("%s: %s", error->name, error->message);
+            ibus_error_free (error);
+        }
+
+        ibus_message_unref (reply);
+    }
+
+    return priv->unique_name;
 }
 
 guint
@@ -556,24 +651,30 @@ ibus_bus_request_name (IBusBus      *bus,
 {
     g_assert (IBUS_IS_BUS (bus));
 
-    guint retval;
-    gboolean result;
+    IBusMessage *reply = NULL;
+    IBusError *error = NULL;
+    guint retval = 0;
 
-    result = ibus_bus_call (bus,
-                            DBUS_SERVICE_DBUS,
-                            DBUS_PATH_DBUS,
-                            DBUS_INTERFACE_DBUS,
-                            "RequestName",
-                            G_TYPE_STRING, &name,
-                            G_TYPE_UINT, &flags,
-                            G_TYPE_INVALID,
-                            G_TYPE_UINT, &retval,
-                            G_TYPE_INVALID);
+    reply = ibus_bus_call_with_reply (bus,
+                                      DBUS_SERVICE_DBUS,
+                                      DBUS_PATH_DBUS,
+                                      DBUS_INTERFACE_DBUS,
+                                      "RequestName",
+                                      G_TYPE_STRING, &name,
+                                      G_TYPE_UINT, &flags,
+                                      G_TYPE_INVALID);
 
-    if (result)
-        return retval;
+    if (reply) {
+        if (!ibus_message_get_args (reply, &error, G_TYPE_UINT, &retval,
+                                    G_TYPE_INVALID)) {
+            g_warning ("%s: %s", error->name, error->message);
+            ibus_error_free (error);
+        }
 
-    return 0;
+        ibus_message_unref (reply);
+    }
+
+    return retval;
 }
 
 guint
@@ -582,46 +683,60 @@ ibus_bus_release_name (IBusBus      *bus,
 {
     g_assert (IBUS_IS_BUS (bus));
 
-    guint retval;
-    gboolean result;
+    IBusMessage *reply = NULL;
+    IBusError *error = NULL;
+    guint retval = 0;
 
-    result = ibus_bus_call (bus,
-                            DBUS_SERVICE_DBUS,
-                            DBUS_PATH_DBUS,
-                            DBUS_INTERFACE_DBUS,
-                            "ReleaseName",
-                            G_TYPE_STRING, &name,
-                            G_TYPE_INVALID,
-                            G_TYPE_UINT, &retval,
-                            G_TYPE_INVALID);
+    reply = ibus_bus_call_with_reply (bus,
+                                      DBUS_SERVICE_DBUS,
+                                      DBUS_PATH_DBUS,
+                                      DBUS_INTERFACE_DBUS,
+                                      "ReleaseName",
+                                      G_TYPE_STRING, &name,
+                                      G_TYPE_INVALID);
 
-    if (result)
-        return retval;
+    if (reply) {
+        if (!ibus_message_get_args (reply, &error, G_TYPE_UINT, &retval,
+                                    G_TYPE_INVALID)) {
+            g_warning ("%s: %s", error->name, error->message);
+            ibus_error_free (error);
+        }
 
-    return 0;
+        ibus_message_unref (reply);
+    }
+
+    return retval;
 }
 
 gboolean
 ibus_bus_name_has_owner (IBusBus        *bus,
                          const gchar    *name)
 {
-    gboolean retval;
-    gboolean result;
+    g_assert (IBUS_IS_BUS (bus));
 
-    result = ibus_bus_call (bus,
-                            DBUS_SERVICE_DBUS,
-                            DBUS_PATH_DBUS,
-                            DBUS_INTERFACE_DBUS,
-                            "NameHasOwner",
-                            G_TYPE_STRING, &name,
-                            G_TYPE_INVALID,
-                            G_TYPE_BOOLEAN, &retval,
-                            G_TYPE_INVALID);
+    IBusMessage *reply = NULL;
+    IBusError *error = NULL;
+    gboolean retval = FALSE;
 
-    if (result)
-        return retval;
+    reply = ibus_bus_call_with_reply (bus,
+                                      DBUS_SERVICE_DBUS,
+                                      DBUS_PATH_DBUS,
+                                      DBUS_INTERFACE_DBUS,
+                                      "NameHasOwner",
+                                      G_TYPE_STRING, &name,
+                                      G_TYPE_INVALID);
 
-    return FALSE;
+    if (reply) {
+        if (!ibus_message_get_args (reply, &error, G_TYPE_BOOLEAN, &retval,
+                                    G_TYPE_INVALID)) {
+            g_warning ("%s: %s", error->name, error->message);
+            ibus_error_free (error);
+        }
+
+        ibus_message_unref (reply);
+    }
+
+    return retval;
 }
 
 GList *
@@ -636,16 +751,13 @@ ibus_bus_add_match (IBusBus     *bus,
 {
     g_assert (IBUS_IS_BUS (bus));
 
-    gboolean result;
-
-    result = ibus_bus_call (bus,
-                            DBUS_SERVICE_DBUS,
-                            DBUS_PATH_DBUS,
-                            DBUS_INTERFACE_DBUS,
-                            "AddMatch",
-                            G_TYPE_STRING, &rule,
-                            G_TYPE_INVALID,
-                            G_TYPE_INVALID);
+    ibus_bus_call (bus,
+                   DBUS_SERVICE_DBUS,
+                   DBUS_PATH_DBUS,
+                   DBUS_INTERFACE_DBUS,
+                   "AddMatch",
+                   G_TYPE_STRING, &rule,
+                   G_TYPE_INVALID);
 }
 
 void
@@ -654,41 +766,46 @@ ibus_bus_remove_match (IBusBus      *bus,
 {
     g_assert (IBUS_IS_BUS (bus));
 
-    gboolean result;
-
-    result = ibus_bus_call (bus,
-                            DBUS_SERVICE_DBUS,
-                            DBUS_PATH_DBUS,
-                            DBUS_INTERFACE_DBUS,
-                            "RemoveMatch",
-                            G_TYPE_STRING, &rule,
-                            G_TYPE_INVALID,
-                            G_TYPE_INVALID);
+    ibus_bus_call (bus,
+                   DBUS_SERVICE_DBUS,
+                   DBUS_PATH_DBUS,
+                   DBUS_INTERFACE_DBUS,
+                   "RemoveMatch",
+                   G_TYPE_STRING, &rule,
+                   G_TYPE_INVALID);
 }
 
-const gchar *
+gchar *
 ibus_bus_get_name_owner (IBusBus        *bus,
                          const gchar    *name)
 {
     g_assert (IBUS_IS_BUS (bus));
 
     gchar *owner = NULL;
-    gboolean result;
+    IBusMessage *reply = NULL;
+    IBusError *error = NULL;
 
-    result = ibus_bus_call (bus,
-                            DBUS_SERVICE_DBUS,
-                            DBUS_PATH_DBUS,
-                            DBUS_INTERFACE_DBUS,
-                            "GetNameOwner",
-                            G_TYPE_STRING, &name,
-                            G_TYPE_INVALID,
-                            G_TYPE_STRING, &owner,
-                            G_TYPE_INVALID);
+    reply = ibus_bus_call_with_reply (bus,
+                                      DBUS_SERVICE_DBUS,
+                                      DBUS_PATH_DBUS,
+                                      DBUS_INTERFACE_DBUS,
+                                      "GetNameOwner",
+                                      G_TYPE_STRING, &name,
+                                      G_TYPE_INVALID);
 
-    if (result)
-        return owner;
+    if (reply) {
+        if (ibus_message_get_args (reply, &error, G_TYPE_STRING, &owner,
+                                   G_TYPE_INVALID)) {
+            owner = g_strdup (owner);
+        } else {
+            g_warning ("%s: %s", error->name, error->message);
+            ibus_error_free (error);
+        }
 
-    return NULL;
+        ibus_message_unref (reply);
+    }
+
+    return owner;
 }
 
 IBusConnection *
@@ -718,7 +835,6 @@ ibus_bus_exit (IBusBus *bus,
                             IBUS_INTERFACE_IBUS,
                             "Exit",
                             G_TYPE_BOOLEAN, &restart,
-                            G_TYPE_INVALID,
                             G_TYPE_INVALID);
     return result;
 }
@@ -738,50 +854,9 @@ ibus_bus_register_component (IBusBus       *bus,
                             IBUS_INTERFACE_IBUS,
                             "RegisterComponent",
                             IBUS_TYPE_COMPONENT, &component,
-                            G_TYPE_INVALID,
                             G_TYPE_INVALID);
 
     return result;
-
-
-#if 0
-    IBusMessage *message, *reply;
-    IBusError *error;
-
-    IBusBusPrivate *priv;
-    priv = IBUS_BUS_GET_PRIVATE (bus);
-
-    message = ibus_message_new_method_call (IBUS_SERVICE_IBUS,
-                                            IBUS_PATH_IBUS,
-                                            IBUS_INTERFACE_IBUS,
-                                            "RegisterComponent");
-
-    ibus_message_append_args (message,
-                              IBUS_TYPE_COMPONENT, &component,
-                              G_TYPE_INVALID);
-
-    reply = ibus_connection_send_with_reply_and_block (
-                                        priv->connection,
-                                        message,
-                                        -1,
-                                        &error);
-    ibus_message_unref (message);
-
-    if (reply == NULL) {
-        g_warning ("%s : %s", error->name, error->message);
-        ibus_error_free (error);
-        return FALSE;
-    }
-
-    if ((error = ibus_error_from_message (reply)) != NULL) {
-        g_warning ("%s : %s", error->name, error->message);
-        ibus_error_free (error);
-        ibus_message_unref (reply);
-        return FALSE;
-    }
-
-    return TRUE;
-#endif
 }
 
 static GList *
@@ -836,20 +911,16 @@ ibus_bus_do_list_engines (IBusBus *bus, gboolean active_engines_only)
         return NULL;
     }
 
-    if (dbus_message_iter_get_array_len (&subiter) <= 0) {
-        ibus_message_unref (reply);
-        return NULL;
-    }
-
     engines = NULL;
-    do {
+    while (ibus_message_iter_get_arg_type (&subiter) != G_TYPE_INVALID) {
         IBusSerializable *object = NULL;
         if (!ibus_message_iter_get (&subiter, IBUS_TYPE_ENGINE_DESC, &object) || !object) {
             g_warning ("Unexpected type is returned from %s", member);
             continue;
         }
         engines = g_list_append (engines, object);
-    } while (ibus_message_iter_next (&subiter));
+        ibus_message_iter_next (&subiter);
+    };
 
     ibus_message_unref (reply);
     return engines;
@@ -898,3 +969,133 @@ ibus_bus_get_config (IBusBus *bus)
 
     return priv->config;
 }
+
+gboolean
+ibus_bus_get_use_sys_layout (IBusBus *bus)
+{
+    g_assert (IBUS_IS_BUS (bus));
+
+    IBusMessage *reply = NULL;
+    IBusError *error = NULL;
+    gboolean use_sys_layout = FALSE;
+
+    reply = ibus_bus_call_with_reply (bus,
+                                      IBUS_SERVICE_IBUS,
+                                      IBUS_PATH_IBUS,
+                                      IBUS_INTERFACE_IBUS,
+                                      "GetUseSysLayout",
+                                      G_TYPE_INVALID);
+    if (reply) {
+        if (!ibus_message_get_args (reply, &error, G_TYPE_BOOLEAN,
+                                    &use_sys_layout, G_TYPE_INVALID)) {
+            g_warning ("%s: %s", error->name, error->message);
+            ibus_error_free (error);
+        }
+
+        ibus_message_unref (reply);
+    }
+
+    return use_sys_layout;
+}
+
+gboolean
+ibus_bus_get_use_global_engine (IBusBus *bus)
+{
+    g_assert (IBUS_IS_BUS (bus));
+
+    IBusMessage *reply = NULL;
+    IBusError *error = NULL;
+    gboolean use_global_engine = FALSE;
+
+    reply = ibus_bus_call_with_reply (bus,
+                                      IBUS_SERVICE_IBUS,
+                                      IBUS_PATH_IBUS,
+                                      IBUS_INTERFACE_IBUS,
+                                      "GetUseGlobalEngine",
+                                      G_TYPE_INVALID);
+    if (reply) {
+        if (!ibus_message_get_args (reply, &error, G_TYPE_BOOLEAN,
+                                    &use_global_engine, G_TYPE_INVALID)) {
+            g_warning ("%s: %s", error->name, error->message);
+            ibus_error_free (error);
+        }
+
+        ibus_message_unref (reply);
+    }
+
+    return use_global_engine;
+}
+
+gboolean
+ibus_bus_is_global_engine_enabled (IBusBus *bus)
+{
+    g_assert (IBUS_IS_BUS (bus));
+
+    IBusMessage *reply = NULL;
+    IBusError *error = NULL;
+    gboolean global_engine_enabled = FALSE;
+
+    reply = ibus_bus_call_with_reply (bus,
+                                      IBUS_SERVICE_IBUS,
+                                      IBUS_PATH_IBUS,
+                                      IBUS_INTERFACE_IBUS,
+                                      "IsGlobalEngineEnabled",
+                                      G_TYPE_INVALID);
+    if (reply) {
+        if (!ibus_message_get_args (reply, &error, G_TYPE_BOOLEAN,
+                                    &global_engine_enabled, G_TYPE_INVALID)) {
+            g_warning ("%s: %s", error->name, error->message);
+            ibus_error_free (error);
+        }
+
+        ibus_message_unref (reply);
+    }
+
+    return global_engine_enabled;
+}
+
+IBusEngineDesc *
+ibus_bus_get_global_engine (IBusBus *bus)
+{
+    g_assert (IBUS_IS_BUS (bus));
+
+    IBusMessage *reply = NULL;
+    IBusError *error = NULL;
+    IBusEngineDesc *global_engine = NULL;
+
+    reply = ibus_bus_call_with_reply (bus,
+                                      IBUS_SERVICE_IBUS,
+                                      IBUS_PATH_IBUS,
+                                      IBUS_INTERFACE_IBUS,
+                                      "GetGlobalEngine",
+                                      G_TYPE_INVALID);
+    if (reply) {
+        if (!ibus_message_get_args (reply, &error, IBUS_TYPE_ENGINE_DESC,
+                                    &global_engine, G_TYPE_INVALID)) {
+            g_warning ("%s: %s", error->name, error->message);
+            ibus_error_free (error);
+        }
+
+        ibus_message_unref (reply);
+    }
+
+    return global_engine;
+}
+
+gboolean
+ibus_bus_set_global_engine (IBusBus     *bus,
+                            const gchar *global_engine)
+{
+    g_assert (IBUS_IS_BUS (bus));
+
+    gboolean result;
+    result = ibus_bus_call (bus,
+                            IBUS_SERVICE_IBUS,
+                            IBUS_PATH_IBUS,
+                            IBUS_INTERFACE_IBUS,
+                            "SetGlobalEngine",
+                            G_TYPE_STRING, &global_engine,
+                            G_TYPE_INVALID);
+
+    return result;
+}
