@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* bus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+#include <dbus/dbus.h>
 #include <glib/gstdio.h>
 #include "ibuscomponent.h"
 
@@ -29,18 +29,19 @@ enum {
 
 /* IBusComponentPriv */
 struct _IBusComponentPrivate {
-    gpointer pad;
+    // TRUE if the component started in the verbose mode.
+    gboolean verbose;
+    // TRUE if the component needs to be restarted when it dies.
+    gboolean restart;
 };
 typedef struct _IBusComponentPrivate IBusComponentPrivate;
 
 #define IBUS_COMPONENT_GET_PRIVATE(o)  \
-   (G_TYPE_INSTANCE_GET_PRIVATE ((o), BUS_TYPE_COMPONENT, IBusComponentPrivate))
+   (G_TYPE_INSTANCE_GET_PRIVATE ((o), IBUS_TYPE_COMPONENT, IBusComponentPrivate))
 
 // static guint            _signals[LAST_SIGNAL] = { 0 };
 
 /* functions prototype */
-static void         ibus_component_class_init   (IBusComponentClass     *klass);
-static void         ibus_component_init         (IBusComponent          *component);
 static void         ibus_component_destroy      (IBusComponent          *component);
 static gboolean     ibus_component_serialize    (IBusComponent          *component,
                                                  IBusMessageIter        *iter);
@@ -60,35 +61,7 @@ static void         ibus_component_parse_observed_paths
                                                  XMLNode                *node,
                                                  gboolean                access_fs);
 
-static IBusSerializableClass  *parent_class = NULL;
-
-GType
-ibus_component_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (IBusComponentClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_component_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (IBusComponent),
-        0,
-        (GInstanceInitFunc) ibus_component_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_SERIALIZABLE,
-                    "IBusComponent",
-                    &type_info,
-                    (GTypeFlags)0);
-    }
-
-    return type;
-}
-
+G_DEFINE_TYPE (IBusComponent, ibus_component, IBUS_TYPE_SERIALIZABLE)
 
 static void
 ibus_component_class_init (IBusComponentClass *klass)
@@ -96,7 +69,7 @@ ibus_component_class_init (IBusComponentClass *klass)
     IBusObjectClass *object_class = IBUS_OBJECT_CLASS (klass);
     IBusSerializableClass *serializable_class = IBUS_SERIALIZABLE_CLASS (klass);
 
-    parent_class = (IBusSerializableClass *) g_type_class_peek_parent (klass);
+    g_type_class_add_private (klass, sizeof (IBusComponentPrivate));
 
     object_class->destroy = (IBusObjectDestroyFunc) ibus_component_destroy;
 
@@ -108,7 +81,6 @@ ibus_component_class_init (IBusComponentClass *klass)
 }
 
 
-
 static void
 ibus_component_init (IBusComponent *component)
 {
@@ -122,6 +94,12 @@ ibus_component_init (IBusComponent *component)
     component->textdomain = NULL;
     component->engines = NULL;
     component->observed_paths = NULL;
+    component->pid = 0;
+    component->child_source_id = 0;
+
+    IBusComponentPrivate * priv = IBUS_COMPONENT_GET_PRIVATE (component);
+    priv->verbose = FALSE;
+    priv->restart = FALSE;
 }
 
 static void
@@ -159,7 +137,18 @@ ibus_component_destroy (IBusComponent *component)
     g_list_free (component->engines);
     component->engines = NULL;
 
-    IBUS_OBJECT_CLASS (parent_class)->destroy (IBUS_OBJECT (component));
+    if (component->pid != 0) {
+        ibus_component_stop (component);
+        g_spawn_close_pid (component->pid);
+        component->pid = 0;
+    }
+
+    if (component->child_source_id != 0) {
+        g_source_remove (component->child_source_id);
+        component->child_source_id = 0;
+    }
+
+    IBUS_OBJECT_CLASS (ibus_component_parent_class)->destroy (IBUS_OBJECT (component));
 }
 
 static gboolean
@@ -170,7 +159,7 @@ ibus_component_serialize (IBusComponent   *component,
     IBusMessageIter array_iter;
     GList *p;
 
-    retval = parent_class->serialize ((IBusSerializable *)component, iter);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_component_parent_class)->serialize ((IBusSerializable *)component, iter);
     g_return_val_if_fail (retval, FALSE);
 
     retval = ibus_message_iter_append (iter, G_TYPE_STRING, &component->name);
@@ -230,7 +219,7 @@ ibus_component_deserialize (IBusComponent   *component,
     gchar *str;
     IBusMessageIter array_iter;
 
-    retval = parent_class->deserialize ((IBusSerializable *)component, iter);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_component_parent_class)->deserialize ((IBusSerializable *)component, iter);
     g_return_val_if_fail (retval, FALSE);
 
     retval = ibus_message_iter_get (iter, G_TYPE_STRING, &str);
@@ -282,6 +271,7 @@ ibus_component_deserialize (IBusComponent   *component,
         g_return_val_if_fail (retval, FALSE);
         ibus_message_iter_next (&array_iter);
 
+        g_object_ref_sink (path);
         component->observed_paths = g_list_append (component->observed_paths, path);
     }
     ibus_message_iter_next (iter);
@@ -309,7 +299,7 @@ ibus_component_copy (IBusComponent       *dest,
 {
     gboolean retval;
 
-    retval = parent_class->copy ((IBusSerializable *)dest,
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_component_parent_class)->copy ((IBusSerializable *)dest,
                                  (IBusSerializable *)src);
     g_return_val_if_fail (retval, FALSE);
 
@@ -354,7 +344,9 @@ ibus_component_output (IBusComponent *component,
 
 #define OUTPUT_ENTRY(field, element)                                        \
     {                                                                       \
-        gchar *escape_text = g_markup_escape_text (component->field, -1);   \
+        gchar *escape_text =                                                \
+            g_markup_escape_text (component->field ?                        \
+                                    component->field : "", -1);             \
         g_string_append_indent (output, indent + 1);                        \
         g_string_append_printf (output, "<"element">%s</"element">\n",      \
                                 escape_text);                               \
@@ -537,6 +529,7 @@ ibus_component_parse_observed_paths (IBusComponent    *component,
         IBusObservedPath *path;
 
         path = ibus_observed_path_new_from_xml_node ((XMLNode *)p->data, access_fs);
+        g_object_ref_sink (path);
         component->observed_paths = g_list_append (component->observed_paths, path);
 
         if (access_fs && path->is_dir && path->is_exist) {
@@ -635,6 +628,7 @@ ibus_component_add_observed_path (IBusComponent *component,
     IBusObservedPath *p;
 
     p = ibus_observed_path_new (path, access_fs);
+    g_object_ref_sink (p);
     component->observed_paths = g_list_append (component->observed_paths, p);
 
     if (access_fs && p->is_dir && p->is_exist) {
@@ -650,6 +644,7 @@ ibus_component_add_engine (IBusComponent  *component,
     g_assert (IBUS_IS_COMPONENT (component));
     g_assert (IBUS_IS_ENGINE_DESC (engine));
 
+    g_object_ref_sink (engine);
     component->engines = g_list_append (component->engines, engine);
     g_object_set_data ((GObject *)engine, "component", component);
 }
@@ -670,6 +665,13 @@ ibus_component_child_cb (GPid            pid,
 
     g_spawn_close_pid (pid);
     component->pid = 0;
+    component->child_source_id = 0;
+
+    IBusComponentPrivate *priv = IBUS_COMPONENT_GET_PRIVATE (component);
+    if (priv->restart) {
+        g_debug ("==== Restarting %s", component->exec);
+        ibus_component_start (component, priv->verbose);
+    }
 }
 
 gboolean
@@ -680,6 +682,9 @@ ibus_component_start (IBusComponent *component, gboolean verbose)
     if (component->pid != 0)
         return TRUE;
 
+    IBusComponentPrivate *priv = IBUS_COMPONENT_GET_PRIVATE (component);
+    priv->verbose = verbose;
+
     gint argc;
     gchar **argv;
     gboolean retval;
@@ -702,15 +707,15 @@ ibus_component_start (IBusComponent *component, gboolean verbose)
                             flags,
                             NULL, NULL,
                             &(component->pid), &error);
-    g_strfreev (argv)
-    ;
+    g_strfreev (argv);
     if (!retval) {
         g_warning ("Can not execute component %s: %s", component->name, error->message);
         g_error_free (error);
         return FALSE;
     }
 
-    g_child_watch_add (component->pid, (GChildWatchFunc) ibus_component_child_cb, component);
+    component->child_source_id =
+        g_child_watch_add (component->pid, (GChildWatchFunc) ibus_component_child_cb, component);
 
     return TRUE;
 }
@@ -761,3 +766,12 @@ ibus_component_get_from_engine (IBusEngineDesc *engine)
     component = (IBusComponent *)g_object_get_data ((GObject *)engine, "component");
     return component;
 }
+
+void
+ibus_component_set_restart (IBusComponent *component, gboolean restart)
+{
+    g_assert (IBUS_IS_COMPONENT (component));
+
+    IBusComponentPrivate *priv = IBUS_COMPONENT_GET_PRIVATE (component);
+    priv->restart = restart;
+}
