@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* bus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,7 @@
 /**
  * SECTION: ibuscomponent
  * @short_description:  Component (executable) specification.
+ * @title: IBusComponent
  * @stability: Stable
  *
  * An IBusComponent is an executable program.
@@ -102,6 +103,10 @@ struct _IBusComponent {
     GList *observed_paths;
 
     GPid     pid;
+    guint    child_source_id;
+
+    /* padding */
+    gpointer pdummy[5];  // We can add 5 pointers without breaking the ABI.
 };
 
 struct _IBusComponentClass {
@@ -183,7 +188,7 @@ void             ibus_component_add_engine      (IBusComponent  *component,
 /**
  * ibus_component_get_engines:
  * @component: An IBusComponent.
- * @returns: A newly allocated GList that contains engines.
+ * @returns: (transfer none) (element-type IBusEngineDesc): A newly allocated GList that contains engines.
  *
  * Get the engines of this component.
  */
@@ -228,7 +233,7 @@ gboolean         ibus_component_check_modification
 /**
  * ibus_component_start:
  * @component: An IBusComponent.
- * @verbose: if redirect the child output to /dev/null
+ * @verbose: if FALSE, redirect the child output to /dev/null
  * @returns: TRUE if the component is started; FALSE otherwise.
  *
  * Whether the IBusComponent is started.
@@ -257,12 +262,23 @@ gboolean         ibus_component_is_running      (IBusComponent  *component);
 /**
  * ibus_component_get_from_engine:
  * @engine: A description of an engine.
- * @returns: An IBusComponent of the engine.
+ * @returns: (transfer none): An IBusComponent of the engine.
  *
  * Get the IBusComponent from an engine description.
  */
 IBusComponent   *ibus_component_get_from_engine (IBusEngineDesc *engine);
 
+/**
+ * ibus_component_set_restart:
+ * @component: An IBusComponent.
+ * @restart: if TRUE, the component will be restartd when it dies.
+ *
+ * Set whether the component needs to be restarted when it dies.
+ */
+void             ibus_component_set_restart     (IBusComponent  *component,
+                                                 gboolean        restart);
+
+
 G_END_DECLS
 #endif
 
