@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+#include <dbus/dbus.h>
 #include "ibusinternal.h"
 #include "ibusmarshalers.h"
 #include "ibusshare.h"
@@ -101,6 +101,7 @@ ibus_config_new (IBusConnection *connection)
     GObject *obj;
     obj = g_object_new (IBUS_TYPE_CONFIG,
                         "name", IBUS_SERVICE_CONFIG,
+                        "interface", IBUS_INTERFACE_CONFIG,
                         "path", IBUS_PATH_CONFIG,
                         "connection", connection,
                         NULL);
@@ -126,6 +127,7 @@ ibus_config_class_init (IBusConfigClass *klass)
     /* install signals */
     /**
      * IBusConfig::value-changed:
+     * @config: An IBusConfig.
      * @section: Section name.
      * @name: Name of the property.
      * @value: Value.
@@ -281,3 +283,23 @@ ibus_config_set_value (IBusConfig   *config,
     g_assert (retval);
     return TRUE;
 }
+
+gboolean
+ibus_config_unset (IBusConfig   *config,
+                   const gchar  *section,
+                   const gchar  *name)
+{
+    g_assert (IBUS_IS_CONFIG (config));
+    g_assert (section != NULL);
+    g_assert (name != NULL);
+
+    gboolean retval;
+
+    retval = ibus_proxy_call ((IBusProxy *) config,
+                              "Unset",
+                              G_TYPE_STRING, &section,
+                              G_TYPE_STRING, &name,
+                              G_TYPE_INVALID);
+    g_assert (retval);
+    return TRUE;
+}
