@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -94,6 +94,35 @@
  *        <para>Get value of a configuration option.
  *        </para>
  *     </listitem>
+ *     <listitem>
+ *         <para>gboolean unset(IBusConfigService *config, const gchar *section, const gchar *name,
+ *             IBusError **error)
+ *         </para>
+ *         <variablelist>
+ *             <varlistentry>
+ *                 <term>config:</term>
+ *                 <listitem>A configure service</listitem>
+ *             </varlistentry>
+ *             <varlistentry>
+ *                 <term>section:</term>
+ *                 <listitem>Section name of the configuration option.</listitem>
+ *             </varlistentry>
+ *             <varlistentry>
+ *                 <term>name:</term>
+ *                 <listitem>Name of the configuration option.</listitem>
+ *             </varlistentry>
+ *             <varlistentry>
+ *                 <term>error:</term>
+ *                 <listitem>Error outputs here.</listitem>
+ *             </varlistentry>
+ *             <varlistentry>
+ *                 <term>Returns:</term>
+ *                 <listitem>TRUE if succeed; FALSE otherwise.</listitem>
+ *             </varlistentry>
+ *         </variablelist>
+ *         <para>Remove an entry to a configuration option.
+ *         </para>
+ *     </listitem>
  * </itemizedlist>
  */
 #ifndef __IBUS_CONFIG_SERVICE_H_
@@ -149,10 +178,14 @@ struct _IBusConfigServiceClass {
                                const gchar          *name,
                                GValue               *value,
                                IBusError           **error);
+    gboolean    (* unset)     (IBusConfigService    *config,
+                               const gchar          *section,
+                               const gchar          *name,
+                               IBusError           **error);
 
     /*< private >*/
     /* padding */
-    gpointer pdummy[14];
+    gpointer pdummy[13];
 };
 
 GType                ibus_config_service_get_type   (void);
