@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,8 +18,8 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
 #include <stdarg.h>
+#include <dbus/dbus.h>
 #include "ibusengine.h"
 #include "ibusinternal.h"
 #include "ibusshare.h"
@@ -64,8 +64,6 @@ typedef struct _IBusEnginePrivate IBusEnginePrivate;
 static guint            engine_signals[LAST_SIGNAL] = { 0 };
 
 /* functions prototype */
-static void     ibus_engine_class_init      (IBusEngineClass    *klass);
-static void     ibus_engine_init            (IBusEngine         *engine);
 static void     ibus_engine_destroy         (IBusEngine         *engine);
 static void     ibus_engine_set_property    (IBusEngine         *engine,
                                              guint               prop_id,
@@ -116,33 +114,7 @@ static void     ibus_engine_property_hide   (IBusEngine         *engine,
                                              const gchar        *prop_name);
 
 
-static IBusServiceClass  *parent_class = NULL;
-
-GType
-ibus_engine_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (IBusEngineClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_engine_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (IBusEngine),
-        0,
-        (GInstanceInitFunc) ibus_engine_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_SERVICE,
-                    "IBusEngine",
-                    &type_info,
-                    (GTypeFlags) 0);
-    }
-    return type;
-}
+G_DEFINE_TYPE (IBusEngine, ibus_engine, IBUS_TYPE_SERVICE)
 
 IBusEngine *
 ibus_engine_new (const gchar    *name,
@@ -169,8 +141,6 @@ ibus_engine_class_init (IBusEngineClass *klass)
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (klass);
 
-    parent_class = (IBusServiceClass *) g_type_class_peek_parent (klass);
-
     g_type_class_add_private (klass, sizeof (IBusEnginePrivate));
 
     gobject_class->set_property = (GObjectSetPropertyFunc) ibus_engine_set_property;
@@ -486,6 +456,9 @@ ibus_engine_class_init (IBusEngineClass *klass)
     /**
      * IBusEngine::candidate-clicked:
      * @engine: An IBusEngine.
+     * @index:  Index of candidate be clicked.
+     * @button: Mouse button.
+     * @state:  Keyboard state.
      *
      * Emitted when candidate on lookup table is clicked.
      * Implement the member function candidate_clicked() in extended class to receive this signal.
@@ -508,6 +481,8 @@ ibus_engine_class_init (IBusEngineClass *klass)
     /**
      * IBusEngine::property-activate:
      * @engine: An IBusEngine.
+     * @name:   Property name.
+     * @state:  Property state.
      *
      * Emitted when a property is activated or change changed.
      * Implement the member function property_activate() in extended class to receive this signal.
@@ -529,6 +504,7 @@ ibus_engine_class_init (IBusEngineClass *klass)
     /**
      * IBusEngine::property-show:
      * @engine: An IBusEngine.
+     * @name:   Property name.
      *
      * Emitted when a property is shown.
      * Implement the member function property_side() in extended class to receive this signal.
@@ -549,6 +525,7 @@ ibus_engine_class_init (IBusEngineClass *klass)
     /**
      * IBusEngine::property-hide:
      * @engine: An IBusEngine.
+     * @name:   Property name.
      *
      * Emitted when a property is hidden.
      * Implement the member function property_hide() in extended class to receive this signal.
@@ -591,7 +568,7 @@ ibus_engine_destroy (IBusEngine *engine)
         priv->connection = NULL;
     }
 
-    IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (engine));
+    IBUS_OBJECT_CLASS(ibus_engine_parent_class)->destroy (IBUS_OBJECT (engine));
 }
 
 static void
@@ -610,7 +587,7 @@ ibus_engine_set_property (IBusEngine   *engine,
 
     case PROP_CONNECTION:
         priv->connection = g_value_get_object (value);
-        g_object_ref (priv->connection);
+        g_object_ref_sink (priv->connection);
         ibus_service_add_to_connection ((IBusService *) engine,
                                         priv->connection);
         break;
@@ -683,7 +660,8 @@ ibus_engine_ibus_message (IBusEngine     *engine,
     name = ibus_message_get_member (message);
 
     if (interface != NULL && g_strcmp0 (interface, IBUS_INTERFACE_ENGINE) != 0)
-        return parent_class->ibus_message ((IBusService *) engine, connection, message);
+        return IBUS_SERVICE_CLASS (ibus_engine_parent_class)->ibus_message (
+                        (IBusService *) engine, connection, message);
 
     do {
         if (g_strcmp0 (name, "ProcessKeyEvent") == 0) {
@@ -720,10 +698,12 @@ ibus_engine_ibus_message (IBusEngine     *engine,
             }
             break;
         }
-        for (i = 0; i < G_N_ELEMENTS (no_arg_methods); i++) {
-            if (g_strcmp0 (name, no_arg_methods[i].member) != 0)
-                continue;
 
+        for (i = 0;
+             i < G_N_ELEMENTS (no_arg_methods) && g_strcmp0 (name, no_arg_methods[i].member) != 0;
+             i++);
+
+        if (i < G_N_ELEMENTS (no_arg_methods)) {
             IBusMessageIter iter;
             ibus_message_iter_init (message, &iter);
             if (ibus_message_iter_has_next (&iter)) {
@@ -736,9 +716,9 @@ ibus_engine_ibus_message (IBusEngine     *engine,
                 g_signal_emit (engine, engine_signals[no_arg_methods[i].signal_id], 0);
                 reply = ibus_message_new_method_return (message);
             }
-        }
-        if (i < G_N_ELEMENTS (no_arg_methods))
             break;
+        }
+
         if (g_strcmp0 (name, "CandidateClicked") == 0) {
             guint index, button, state;
 
@@ -903,6 +883,7 @@ ibus_engine_ibus_message (IBusEngine     *engine,
                         DBUS_ERROR_UNKNOWN_METHOD,
                         "%s.%s",
                         IBUS_INTERFACE_ENGINE, name);
+            g_warn_if_reached ();
         }
     } while (0);
 
@@ -1055,6 +1036,10 @@ ibus_engine_commit_text (IBusEngine *engine,
                   "CommitText",
                   IBUS_TYPE_TEXT, &text,
                   G_TYPE_INVALID);
+
+    if (g_object_is_floating (text)) {
+        g_object_unref (text);
+    }
 }
 
 void
@@ -1062,13 +1047,29 @@ ibus_engine_update_preedit_text (IBusEngine      *engine,
                                  IBusText        *text,
                                  guint            cursor_pos,
                                  gboolean         visible)
+{
+    ibus_engine_update_preedit_text_with_mode (engine,
+            text, cursor_pos, visible, IBUS_ENGINE_PREEDIT_CLEAR);
+}
+
+void
+ibus_engine_update_preedit_text_with_mode (IBusEngine            *engine,
+                                           IBusText              *text,
+                                           guint                  cursor_pos,
+                                           gboolean               visible,
+                                           IBusPreeditFocusMode   mode)
 {
     _send_signal (engine,
                   "UpdatePreeditText",
                   IBUS_TYPE_TEXT, &text,
                   G_TYPE_UINT, &cursor_pos,
                   G_TYPE_BOOLEAN, &visible,
+                  G_TYPE_UINT, &mode,
                   G_TYPE_INVALID);
+
+    if (g_object_is_floating (text)) {
+        g_object_unref (text);
+    }
 }
 
 void
@@ -1095,6 +1096,10 @@ void ibus_engine_update_auxiliary_text (IBusEngine      *engine,
                   IBUS_TYPE_TEXT, &text,
                   G_TYPE_BOOLEAN, &visible,
                   G_TYPE_INVALID);
+
+    if (g_object_is_floating (text)) {
+        g_object_unref (text);
+    }
 }
 
 void
@@ -1123,6 +1128,10 @@ ibus_engine_update_lookup_table (IBusEngine        *engine,
                   IBUS_TYPE_LOOKUP_TABLE, &table,
                   G_TYPE_BOOLEAN, &visible,
                   G_TYPE_INVALID);
+
+    if (g_object_is_floating (table)) {
+        g_object_unref (table);
+    }
 }
 
 void
@@ -1131,6 +1140,7 @@ ibus_engine_update_lookup_table_fast (IBusEngine        *engine,
                                       gboolean           visible)
 {
     IBusLookupTable *new_table;
+    IBusText *text;
     gint page_begin;
     gint i;
 
@@ -1147,11 +1157,18 @@ ibus_engine_update_lookup_table_fast (IBusEngine        *engine,
         ibus_lookup_table_append_candidate (new_table, ibus_lookup_table_get_candidate (table, i));
     }
 
+    for (i = 0; (text = ibus_lookup_table_get_label (table, i)) != NULL; i++) {
+        ibus_lookup_table_append_label (new_table, text);
+    }
+
     ibus_lookup_table_set_cursor_pos (new_table, ibus_lookup_table_get_cursor_in_page (table));
+    ibus_lookup_table_set_orientation (new_table, ibus_lookup_table_get_orientation (table));
 
     ibus_engine_update_lookup_table (engine, new_table, visible);
 
-    g_object_unref (new_table);
+    if (g_object_is_floating (table)) {
+        g_object_unref (table);
+    }
 }
 
 void ibus_engine_show_lookup_table (IBusEngine *engine)
@@ -1200,6 +1217,10 @@ ibus_engine_register_properties (IBusEngine   *engine,
                   "RegisterProperties",
                   IBUS_TYPE_PROP_LIST, &prop_list,
                   G_TYPE_INVALID);
+
+    if (g_object_is_floating (prop_list)) {
+        g_object_unref (prop_list);
+    }
 }
 
 void
@@ -1210,6 +1231,10 @@ ibus_engine_update_property (IBusEngine   *engine,
                   "UpdateProperty",
                   IBUS_TYPE_PROPERTY, &prop,
                   G_TYPE_INVALID);
+
+    if (g_object_is_floating (prop)) {
+        g_object_unref (prop);
+    }
 }
 
 const gchar *
