@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,20 +21,21 @@
 /**
  * SECTION: ibusengine
  * @short_description: Input method engine abstract.
+ * @title: IBusEngine
  * @stability: Stable
- * @see_also: #IBusComponent, #IBusEngineDesc
  *
  * An IBusEngine provides infrastructure for input method engine.
  * Developers can "extend" this class for input method engine development.
+ *
+ * @see_also: #IBusComponent, #IBusEngineDesc
  */
 #ifndef __IBUS_ENGINE_H_
 #define __IBUS_ENGINE_H_
 
-#include <dbus/dbus.h>
 #include "ibusservice.h"
 #include "ibusattribute.h"
 #include "ibuslookuptable.h"
-#include "ibusproperty.h"
+#include "ibusproplist.h"
 
 /*
  * Type macros.
@@ -149,6 +150,10 @@ IBusEngine  *ibus_engine_new            (const gchar        *name,
  * @text: String commit to IBusEngine.
  *
  * Commit output of input method to IBus client.
+ *
+ * (Note: The text object will be released, if it is floating.
+ *  If caller want to keep the object, caller should make the object
+ *  sink by g_object_ref_sink.)
  */
 void         ibus_engine_commit_text    (IBusEngine         *engine,
                                          IBusText           *text);
@@ -161,6 +166,10 @@ void         ibus_engine_commit_text    (IBusEngine         *engine,
  * @visible: Whether the pre-edit buffer is visible.
  *
  * Update the pre-edit buffer.
+ *
+ * (Note: The text object will be released, if it is floating.
+ *  If caller want to keep the object, caller should make the object
+ *  sink by g_object_ref_sink.)
  */
 void         ibus_engine_update_preedit_text
                                         (IBusEngine         *engine,
@@ -168,6 +177,30 @@ void         ibus_engine_update_preedit_text
                                          guint               cursor_pos,
                                          gboolean            visible);
 
+/**
+ * ibus_engine_update_preedit_text_with_mode:
+ * @engine: An IBusEngine.
+ * @text: Update content.
+ * @cursor_pos: Current position of cursor
+ * @visible: Whether the pre-edit buffer is visible.
+ * @mode: Pre-edit commit mode when the focus is lost.
+ *
+ * Update the pre-edit buffer with commit mode.
+ * if mode is IBUS_ENGINE_PREEDIT_CLEAR,
+ * ibus_engine_update_preedit_text_with_mode is compatible with
+ * ibus_engine_update_preedit_text.
+ *
+ * (Note: The text object will be released, if it is floating.
+ *  If caller want to keep the object, caller should make the object
+ *  sink by g_object_ref_sink.)
+ */
+void         ibus_engine_update_preedit_text_with_mode
+                                        (IBusEngine              *engine,
+                                         IBusText                *text,
+                                         guint                    cursor_pos,
+                                         gboolean                 visible,
+                                         IBusPreeditFocusMode     mode);
+
 /**
  * ibus_engine_show_preedit_text:
  * @engine: An IBusEngine.
@@ -193,6 +226,10 @@ void         ibus_engine_hide_preedit_text
  * @visible: Whether the auxiliary text bar is visible.
  *
  * Update the auxiliary bar.
+ *
+ * (Note: The text object will be released, if it is floating.
+ *  If caller want to keep the object, caller should make the object
+ *  sink by g_object_ref_sink.)
  */
 void         ibus_engine_update_auxiliary_text
                                         (IBusEngine        *engine,
@@ -224,6 +261,10 @@ void         ibus_engine_hide_auxiliary_text
  * @visible: Whether the lookup_table is visible.
  *
  * Update the lookup table.
+ *
+ * (Note: The table object will be released, if it is floating.
+ *  If caller want to keep the object, caller should make the object
+ *  sink by g_object_ref_sink.)
  */
 void         ibus_engine_update_lookup_table
                                         (IBusEngine         *engine,
@@ -241,6 +282,9 @@ void         ibus_engine_update_lookup_table
  * If size of lookup table is not over table page size *4,
  * then it calls ibus_engine_update_lookup_table().
  *
+ * (Note: The table object will be released, if it is floating.
+ *  If caller want to keep the object, caller should make the object
+ *  sink by g_object_ref_sink.)
  */
 void         ibus_engine_update_lookup_table_fast
                                         (IBusEngine         *engine,
@@ -286,6 +330,10 @@ void         ibus_engine_forward_key_event
  * @prop_list: Property List.
  *
  * Register and show properties in language bar.
+ *
+ * (Note: The prop_list object will be released, if it is floating.
+ *  If caller want to keep the object, caller should make the object
+ *  sink by g_object_ref_sink.)
  */
 void         ibus_engine_register_properties
                                         (IBusEngine         *engine,
@@ -297,10 +345,26 @@ void         ibus_engine_register_properties
  * @prop: IBusProperty to be updated.
  *
  * Update the state displayed in language bar.
+ *
+ * (Note: The prop object will be released, if it is floating.
+ *  If caller want to keep the object, caller should make the object
+ *  sink by g_object_ref_sink.)
  */
 void         ibus_engine_update_property(IBusEngine         *engine,
                                          IBusProperty       *prop);
 
+/**
+ * ibus_engine_delete_surrounding_text:
+ * @engine: An IBusEngine.
+ * @offset: The offset of the first char.
+ * @nchars: Number of chars to be deleted.
+ *
+ * Delete surrounding text.
+ */
+void ibus_engine_delete_surrounding_text(IBusEngine         *engine,
+                                         gint                offset,
+                                         guint               nchars);
+
 /**
  * ibus_engine_get_name:
  * @engine: An IBusEngine.
