@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* bus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -39,8 +39,6 @@ typedef struct _IBusEngineDescPrivate IBusEngineDescPrivate;
 // static guint            _signals[LAST_SIGNAL] = { 0 };
 
 /* functions prototype */
-static void         ibus_engine_desc_class_init     (IBusEngineDescClass    *klass);
-static void         ibus_engine_desc_init           (IBusEngineDesc         *desc);
 static void         ibus_engine_desc_destroy        (IBusEngineDesc         *desc);
 static gboolean     ibus_engine_desc_serialize      (IBusEngineDesc         *desc,
                                                      IBusMessageIter        *iter);
@@ -51,34 +49,7 @@ static gboolean     ibus_engine_desc_copy           (IBusEngineDesc         *des
 static gboolean     ibus_engine_desc_parse_xml_node (IBusEngineDesc         *desc,
                                                      XMLNode                *node);
 
-static IBusSerializableClass  *parent_class = NULL;
-
-GType
-ibus_engine_desc_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (IBusEngineDescClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_engine_desc_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (IBusEngineDesc),
-        0,
-        (GInstanceInitFunc) ibus_engine_desc_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_SERIALIZABLE,
-                    "IBusEngineDesc",
-                    &type_info,
-                    (GTypeFlags)0);
-    }
-
-    return type;
-}
+G_DEFINE_TYPE (IBusEngineDesc, ibus_engine_desc, IBUS_TYPE_SERIALIZABLE)
 
 
 static void
@@ -87,15 +58,13 @@ ibus_engine_desc_class_init (IBusEngineDescClass *klass)
     IBusObjectClass *object_class = IBUS_OBJECT_CLASS (klass);
     IBusSerializableClass *serializable_class = IBUS_SERIALIZABLE_CLASS (klass);
 
-    parent_class = (IBusSerializableClass *) g_type_class_peek_parent (klass);
-
     object_class->destroy = (IBusObjectDestroyFunc) ibus_engine_desc_destroy;
 
     serializable_class->serialize   = (IBusSerializableSerializeFunc) ibus_engine_desc_serialize;
     serializable_class->deserialize = (IBusSerializableDeserializeFunc) ibus_engine_desc_deserialize;
     serializable_class->copy        = (IBusSerializableCopyFunc) ibus_engine_desc_copy;
 
-    g_string_append (serializable_class->signature, "ssssssss");
+    g_string_append (serializable_class->signature, "sssssssssu");
 }
 
 static void
@@ -110,6 +79,7 @@ ibus_engine_desc_init (IBusEngineDesc *desc)
     desc->author = NULL;
     desc->icon = NULL;
     desc->layout = NULL;
+    desc->hotkeys = NULL;
     desc->rank = 0;
 }
 
@@ -124,8 +94,9 @@ ibus_engine_desc_destroy (IBusEngineDesc *desc)
     g_free (desc->author);
     g_free (desc->icon);
     g_free (desc->layout);
+    g_free (desc->hotkeys);
 
-    IBUS_OBJECT_CLASS (parent_class)->destroy (IBUS_OBJECT (desc));
+    IBUS_OBJECT_CLASS (ibus_engine_desc_parent_class)->destroy (IBUS_OBJECT (desc));
 }
 
 static gboolean
@@ -134,7 +105,7 @@ ibus_engine_desc_serialize (IBusEngineDesc  *desc,
 {
     gboolean retval;
 
-    retval = parent_class->serialize ((IBusSerializable *)desc, iter);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_engine_desc_parent_class)->serialize ((IBusSerializable *)desc, iter);
     g_return_val_if_fail (retval, FALSE);
 
     retval = ibus_message_iter_append (iter, G_TYPE_STRING, &desc->name);
@@ -161,6 +132,12 @@ ibus_engine_desc_serialize (IBusEngineDesc  *desc,
     retval = ibus_message_iter_append (iter, G_TYPE_STRING, &desc->layout);
     g_return_val_if_fail (retval, FALSE);
 
+    retval = ibus_message_iter_append (iter, G_TYPE_STRING, &desc->hotkeys);
+    g_return_val_if_fail (retval, FALSE);
+
+    retval = ibus_message_iter_append (iter, G_TYPE_UINT, &desc->rank);
+    g_return_val_if_fail (retval, FALSE);
+
     return TRUE;
 }
 
@@ -171,7 +148,7 @@ ibus_engine_desc_deserialize (IBusEngineDesc  *desc,
     gboolean retval;
     gchar *str;
 
-    retval = parent_class->deserialize ((IBusSerializable *)desc, iter);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_engine_desc_parent_class)->deserialize ((IBusSerializable *)desc, iter);
     g_return_val_if_fail (retval, FALSE);
 
     retval = ibus_message_iter_get (iter, G_TYPE_STRING, &str);
@@ -214,6 +191,15 @@ ibus_engine_desc_deserialize (IBusEngineDesc  *desc,
     ibus_message_iter_next (iter);
     desc->layout = g_strdup (str);
 
+    retval = ibus_message_iter_get (iter, G_TYPE_STRING, &str);
+    g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
+    desc->hotkeys = g_strdup (str);
+
+    retval = ibus_message_iter_get (iter, G_TYPE_UINT, &desc->rank);
+    g_return_val_if_fail (retval, FALSE);
+    ibus_message_iter_next (iter);
+
     return TRUE;
 }
 
@@ -223,7 +209,7 @@ ibus_engine_desc_copy (IBusEngineDesc       *dest,
 {
     gboolean retval;
 
-    retval = parent_class->copy ((IBusSerializable *)dest,
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_engine_desc_parent_class)->copy ((IBusSerializable *)dest,
                                  (IBusSerializable *)src);
     g_return_val_if_fail (retval, FALSE);
 
@@ -236,6 +222,7 @@ ibus_engine_desc_copy (IBusEngineDesc       *dest,
     dest->author        = g_strdup (src->author);
     dest->icon          = g_strdup (src->icon);
     dest->layout        = g_strdup (src->layout);
+    dest->hotkeys       = g_strdup (src->hotkeys);
 
     return TRUE;
 }
@@ -272,6 +259,7 @@ ibus_engine_desc_output (IBusEngineDesc *desc,
     OUTPUT_ENTRY_1(author);
     OUTPUT_ENTRY_1(icon);
     OUTPUT_ENTRY_1(layout);
+    OUTPUT_ENTRY_1(hotkeys);
     g_string_append_indent (output, indent + 1);
     g_string_append_printf (output, "<rank>%u</rank>\n", desc->rank);
 #undef OUTPUT_ENTRY
@@ -303,8 +291,9 @@ ibus_engine_desc_parse_xml_node (IBusEngineDesc *desc,
         PARSE_ENTRY_1(author);
         PARSE_ENTRY_1(icon);
         PARSE_ENTRY_1(layout);
+        PARSE_ENTRY_1(hotkeys);
 #undef PARSE_ENTRY
-#undef PARSE_ENTRY1
+#undef PARSE_ENTRY_1
         if (g_strcmp0 (sub_node->name , "rank") == 0) {
             desc->rank = atoi (sub_node->text);
             continue;
@@ -323,6 +312,21 @@ ibus_engine_desc_new (const gchar *name,
                       const gchar *author,
                       const gchar *icon,
                       const gchar *layout)
+{
+    return ibus_engine_desc_new2 (name, longname, description, language,
+                                  license, author, icon, layout, "");
+}
+
+IBusEngineDesc *
+ibus_engine_desc_new2 (const gchar *name,
+                       const gchar *longname,
+                       const gchar *description,
+                       const gchar *language,
+                       const gchar *license,
+                       const gchar *author,
+                       const gchar *icon,
+                       const gchar *layout,
+                       const gchar *hotkeys)
 {
     g_assert (name);
     g_assert (longname);
@@ -332,6 +336,7 @@ ibus_engine_desc_new (const gchar *name,
     g_assert (author);
     g_assert (icon);
     g_assert (layout);
+    g_assert (hotkeys);
 
     IBusEngineDesc *desc;
     desc = (IBusEngineDesc *)g_object_new (IBUS_TYPE_ENGINE_DESC, NULL);
@@ -344,6 +349,7 @@ ibus_engine_desc_new (const gchar *name,
     desc->author        = g_strdup (author);
     desc->icon          = g_strdup (icon);
     desc->layout        = g_strdup (layout);
+    desc->hotkeys       = g_strdup (hotkeys);
 
     return desc;
 }
@@ -368,4 +374,3 @@ ibus_engine_desc_new_from_xml_node (XMLNode      *node)
 
     return desc;
 }
-
