@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* bus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,8 +21,8 @@
 /**
  * SECTION: ibusenginedesc
  * @short_description:  Input method engine description data.
+ * @title: IBusEngineDesc
  * @stability: Stable
- * @see_also: #IBusComponent, #IBusEngine
  *
  * An IBusEngineDesc stores description data of IBusEngine.
  * The description data can either be passed to ibus_engine_desc_new(),
@@ -32,6 +32,9 @@
  * However, the recommended way to load engine description data is
  * using ibus_component_new_from_file() to load a component file,
  * which also includes engine description data.
+ *
+ * @see_also: #IBusComponent, #IBusEngine
+ *
  */
 
 #ifndef __ENGINE_DESC_H_
@@ -74,6 +77,8 @@ typedef struct _BusComponent BusComponent;
  * @author: Author of the input method engine.
  * @icon: Icon file of this engine.
  * @layout: Keyboard layout
+ * @hotkeys: One or more hotkeys for switching to this engine, separated by
+ *  semi-colon.
  * @rank: Preference rank among engines, the highest ranked IME will put in
  * the front.
  *
@@ -92,6 +97,7 @@ struct _IBusEngineDesc {
     gchar *author;
     gchar *icon;
     gchar *layout;
+    gchar *hotkeys;
     guint  rank;
 };
 
@@ -124,6 +130,32 @@ IBusEngineDesc  *ibus_engine_desc_new           (const gchar    *name,
                                                  const gchar    *author,
                                                  const gchar    *icon,
                                                  const gchar    *layout);
+
+/**
+ * ibus_engine_desc_new2:
+ * @name: Name of the engine.
+ * @longname: Long name of the input method engine.
+ * @description: Input method engine description.
+ * @language: Language (e.g. zh, jp) supported by this input method engine.
+ * @license: License of the input method engine.
+ * @author: Author of the input method engine.
+ * @icon: Icon file of this engine.
+ * @layout: Keyboard layout
+ * @hotkeys: Hotkeys for switching to this engine.
+ * @returns: A newly allocated IBusEngineDesc.
+ *
+ * New a IBusEngineDesc.
+ */
+IBusEngineDesc  *ibus_engine_desc_new2          (const gchar    *name,
+                                                 const gchar    *longname,
+                                                 const gchar    *description,
+                                                 const gchar    *language,
+                                                 const gchar    *license,
+                                                 const gchar    *author,
+                                                 const gchar    *icon,
+                                                 const gchar    *layout,
+                                                 const gchar    *hotkeys);
+
 /**
  * ibus_engine_desc_new_from_xml_node:
  * @node: An XML node
@@ -146,9 +178,8 @@ IBusEngineDesc  *ibus_engine_desc_new_from_xml_node
  * Output XML-formatted input method engine description.
  * The result will be append to GString specified in @output.
  */
-void             ibus_engine_desc_output        (IBusEngineDesc  *info,
+void             ibus_engine_desc_output        (IBusEngineDesc *info,
                                                  GString        *output,
                                                  gint            indent);
 G_END_DECLS
 #endif
-
