@@ -127,6 +127,21 @@ GType ibus_capabilite_get_type (void) G_GNUC_CONST;
  * Returns the type of IBUS_CAPABILITE as GType.
  */
 #define IBUS_TYPE_CAPABILITE (ibus_capabilite_get_type ())
+/**
+ * ibus_preedit_focus_mode_get_type:
+ * @returns: GType of IBUS_PREEDIT_FOCUS_MODE
+ *
+ * Returns the type of IBUS_PREEDIT_FOCUS_MODE as GType.
+ */
+GType ibus_preedit_focus_mode_get_type (void) G_GNUC_CONST;
+
+/**
+ * IBUS_TYPE_PREEDIT_FOCUS_MODE:
+ * @returns: GType of IBUS_PREEDIT_FOCUS_MODE
+ *
+ * Returns the type of IBUS_PREEDIT_FOCUS_MODE as GType.
+ */
+#define IBUS_TYPE_PREEDIT_FOCUS_MODE (ibus_preedit_focus_mode_get_type ())
 /**
  * ibus_orientation_get_type:
  * @returns: GType of IBUS_ORIENTATION
