@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,7 +30,7 @@
 #define __IBUS_ERROR_H_
 
 #include <glib.h>
-#include <dbus/dbus.h>
+#include "ibusdbus.h"
 
 G_BEGIN_DECLS
 
@@ -81,7 +81,7 @@ IBusError       *ibus_error_new_from_printf (const gchar    *name,
  * @message: A DBusMessage
  * @returns: A newly allocated IBusError.
  *
- * New an IBusError from a #DBusMessage.
+ * New an IBusError from a #IBusMessage.
  */
 IBusError       *ibus_error_new_from_message
                                             (DBusMessage    *message);
