@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* IBus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -160,6 +160,18 @@ GQuark           ibus_hotkey_profile_filter_key_event
                                                  guint               prev_modifiers,
                                                  gpointer            user_data);
 
+/**
+ * ibus_hotkey_profile_lookup_hotkey:
+ * @profile: An IBusHotkeyProfile.
+ * @keyval: Keycode of the hotkey.
+ * @modifiers: Modifiers of the hotkey.
+ * @returns: The event associated to the hotkey or 0 if the hotkey is not in the
+ * profile.
+ */
+GQuark           ibus_hotkey_profile_lookup_hotkey
+                                                (IBusHotkeyProfile  *profile,
+                                                 guint               keyval,
+                                                 guint               modifiers);
+
 G_END_DECLS
 #endif
-
