@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,12 +18,13 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#include <dbus/dbus.h>
 #include "ibusshare.h"
 #include "ibusinternal.h"
 #include "ibusinputcontext.h"
 #include "ibusattribute.h"
 #include "ibuslookuptable.h"
-#include "ibusproperty.h"
+#include "ibusproplist.h"
 
 #define IBUS_INPUT_CONTEXT_GET_PRIVATE(o)  \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), IBUS_TYPE_INPUT_CONTEXT, IBusInputContextPrivate))
@@ -63,39 +64,11 @@ static guint            context_signals[LAST_SIGNAL] = { 0 };
 // static guint            context_signals[LAST_SIGNAL] = { 0 };
 
 /* functions prototype */
-static void     ibus_input_context_class_init   (IBusInputContextClass  *klass);
-static void     ibus_input_context_init         (IBusInputContext       *context);
 static void     ibus_input_context_real_destroy (IBusInputContext       *context);
 static gboolean ibus_input_context_ibus_signal  (IBusProxy              *proxy,
                                                  DBusMessage            *message);
 
-static IBusProxyClass  *parent_class = NULL;
-
-GType
-ibus_input_context_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (IBusInputContextClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_input_context_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (IBusInputContext),
-        0,
-        (GInstanceInitFunc) ibus_input_context_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_PROXY,
-                    "IBusInputContext",
-                    &type_info,
-                    (GTypeFlags)0);
-    }
-    return type;
-}
+G_DEFINE_TYPE (IBusInputContext, ibus_input_context, IBUS_TYPE_PROXY)
 
 IBusInputContext *
 ibus_input_context_new (const gchar     *path,
@@ -107,6 +80,7 @@ ibus_input_context_new (const gchar     *path,
 
     obj = g_object_new (IBUS_TYPE_INPUT_CONTEXT,
                         "name", IBUS_SERVICE_IBUS,
+                        "interface", IBUS_INTERFACE_INPUT_CONTEXT,
                         "path", path,
                         "connection", connection,
                         NULL);
@@ -133,9 +107,6 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (klass);
     IBusProxyClass *proxy_class = IBUS_PROXY_CLASS (klass);
 
-
-    parent_class = (IBusProxyClass *) g_type_class_peek_parent (klass);
-
     g_type_class_add_private (klass, sizeof (IBusInputContextPrivate));
 
     ibus_object_class->destroy = (IBusObjectDestroyFunc) ibus_input_context_real_destroy;
@@ -179,6 +150,10 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
      * @text: Text to be committed.
      *
      * Emitted when the text is going to be committed.
+     *
+     * (Note: The text object is floating, and it will be released after the signal.
+     *  If singal handler want to keep the object, the handler should use g_object_ref_sink()
+     *  to get the ownership of the object.)
      */
     context_signals[COMMIT_TEXT] =
         g_signal_new (I_("commit-text"),
@@ -242,6 +217,10 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
      * @visible: Whether the update is visible.
      *
      * Emitted to update preedit text.
+     *
+     * (Note: The text object is floating, and it will be released after the signal.
+     *  If singal handler want to keep the object, the handler should use g_object_ref_sink()
+     *  to get the ownership of the object.)
      */
     context_signals[UPDATE_PREEDIT_TEXT] =
         g_signal_new (I_("update-preedit-text"),
@@ -291,6 +270,10 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
      * @context: An IBusInputContext.
      *
      * Emitted to hide auxilary text.
+     *
+     * (Note: The text object is floating, and it will be released after the signal.
+     *  If singal handler want to keep the object, the handler should use g_object_ref_sink()
+     *  to get the ownership of the object.)
      */
     context_signals[UPDATE_AUXILIARY_TEXT] =
         g_signal_new (I_("update-auxiliary-text"),
@@ -340,6 +323,10 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
      * @visible: Whether the table should be visible.
      *
      * Emitted to update lookup table.
+     *
+     * (Note: The table object is floating, and it will be released after the signal.
+     *  If singal handler want to keep the object, the handler should use g_object_ref_sink()
+     *  to get the ownership of the object.)
      */
     context_signals[UPDATE_LOOKUP_TABLE] =
         g_signal_new (I_("update-lookup-table"),
@@ -449,6 +436,10 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
      * @props: An IBusPropList that contains properties.
      *
      * Emitted to register the properties in @props.
+     *
+     * (Note: The props object is floating, and it will be released after the signal.
+     *  If singal handler want to keep the object, the handler should use g_object_ref_sink()
+     *  to get the ownership of the object.)
      */
     context_signals[REGISTER_PROPERTIES] =
         g_signal_new (I_("register-properties"),
@@ -467,6 +458,10 @@ ibus_input_context_class_init (IBusInputContextClass *klass)
      * @prop: The IBusProperty to be updated.
      *
      * Emitted to update the property @prop.
+     *
+     * (Note: The prop object is floating, and it will be released after the signal.
+     *  If singal handler want to keep the object, the handler should use g_object_ref_sink()
+     *  to get the ownership of the object.)
      */
     context_signals[UPDATE_PROPERTY] =
         g_signal_new (I_("update-property"),
@@ -500,7 +495,7 @@ ibus_input_context_real_destroy (IBusInputContext *context)
                          G_TYPE_INVALID);
     }
 
-    IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (context));
+    IBUS_OBJECT_CLASS(ibus_input_context_parent_class)->destroy (IBUS_OBJECT (context));
 }
 
 static gboolean
@@ -557,7 +552,8 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
                                             G_TYPE_INVALID);
             if (retval) {
                 g_signal_emit (context, context_signals[COMMIT_TEXT], 0, text);
-                g_object_unref (text);
+                if (g_object_is_floating (text))
+                    g_object_unref (text);
             }
             break;
         }
@@ -581,20 +577,21 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
                                text,
                                cursor_pos,
                                visible);
-                g_object_unref (text);
+                if (g_object_is_floating (text))
+                    g_object_unref (text);
             }
             break;
         }
 
-        for (i = 0; i < G_N_ELEMENTS (signals); i++) {
-            if (g_strcmp0 (name, signals[i].member) == 0) {
-                g_signal_emit (context, context_signals[signals[i].signal_id], 0);
-                break;
-            }
+        for (i = 0;
+             i < G_N_ELEMENTS (signals) && g_strcmp0 (name, signals[i].member) != 0;
+             i++);
+
+        if (i < G_N_ELEMENTS (signals)) {
+            g_signal_emit (context, context_signals[signals[i].signal_id], 0);
+            break;
         }
 
-        if (i < G_N_ELEMENTS (signals))
-            break;
         if (g_strcmp0 (name, "UpdateAuxiliaryText") == 0) {
             IBusText *text;
             gboolean visible;
@@ -612,7 +609,8 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
                                0,
                                text,
                                visible);
-                g_object_unref (text);
+                if (g_object_is_floating (text))
+                    g_object_unref (text);
             }
         }
         else if (g_strcmp0 (name, "UpdateLookupTable") == 0) {
@@ -632,7 +630,8 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
                                0,
                                table,
                                visible);
-                g_object_unref (table);
+                if (g_object_is_floating (table))
+                    g_object_unref (table);
             }
         }
         else if (g_strcmp0 (name, "RegisterProperties") == 0) {
@@ -649,7 +648,8 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
                                context_signals[REGISTER_PROPERTIES],
                                0,
                                prop_list);
-                g_object_unref (prop_list);
+                if (g_object_is_floating (prop_list))
+                    g_object_unref (prop_list);
             }
         }
         else if (g_strcmp0 (name, "UpdateProperty") == 0) {
@@ -662,7 +662,8 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
                                             G_TYPE_INVALID);
             if (retval) {
                 g_signal_emit (context, context_signals[UPDATE_PROPERTY], 0, prop);
-                g_object_unref (prop);
+                if (g_object_is_floating (prop))
+                    g_object_unref (prop);
             }
         }
         else if (g_strcmp0 (name, "ForwardKeyEvent") == 0) {
@@ -680,6 +681,8 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
                                             G_TYPE_INVALID);
 
             if (retval) {
+                /* Forward key event back with IBUS_FORWARD_MASK. And process_key_event will
+                 * not process key event with IBUS_FORWARD_MASK again. */
                 g_signal_emit (context,
                                context_signals[FORWARD_KEY_EVENT],
                                0,
@@ -725,6 +728,61 @@ ibus_input_context_ibus_signal (IBusProxy           *proxy,
     return FALSE;
 }
 
+typedef struct {
+    IBusInputContext *context;
+    guint32 keyval;
+    guint32 keycode;
+    guint32 state;
+} CallData;
+
+static void
+_process_key_event_reply_cb (IBusPendingCall *pending,
+                             CallData        *call_data)
+{
+    IBusMessage *reply_message;
+    IBusError *error;
+    gboolean retval = FALSE;
+
+    reply_message = dbus_pending_call_steal_reply (pending);
+
+    if (reply_message == NULL) {
+        /* reply timeout */
+        retval = FALSE;
+    }
+    else if ((error = ibus_error_new_from_message (reply_message)) != NULL) {
+        /* some error happens */
+        g_warning ("%s: %s", error->name, error->message);
+        ibus_error_free (error);
+        retval = FALSE;
+    }
+    else if (!ibus_message_get_args (reply_message,
+                                    &error,
+                                     G_TYPE_BOOLEAN, &retval,
+                                     G_TYPE_INVALID)) {
+        /* can not get return value */
+        g_warning ("%s: %s", error->name, error->message);
+        ibus_error_free (error);
+        retval = FALSE;
+    }
+
+    if (!retval) {
+        /* Forward key event back with IBUS_FORWARD_MASK. And process_key_event will
+         * not process key event with IBUS_FORWARD_MASK again. */
+        g_signal_emit (call_data->context,
+                       context_signals[FORWARD_KEY_EVENT],
+                       0,
+                       call_data->keyval,
+                       call_data->keycode,
+                       call_data->state | IBUS_FORWARD_MASK);
+    }
+}
+
+static void
+_call_data_free (CallData *call_data)
+{
+    g_object_unref (call_data->context);
+    g_slice_free (CallData, call_data);
+}
 
 gboolean
 ibus_input_context_process_key_event (IBusInputContext *context,
@@ -734,9 +792,9 @@ ibus_input_context_process_key_event (IBusInputContext *context,
 {
     g_assert (IBUS_IS_INPUT_CONTEXT (context));
 
-    IBusMessage *reply_message;
     IBusPendingCall *pending = NULL;
     IBusError *error = NULL;
+    CallData *call_data;
     gboolean retval;
 
     if (state & IBUS_HANDLED_MASK)
@@ -760,37 +818,27 @@ ibus_input_context_process_key_event (IBusInputContext *context,
         return FALSE;
     }
 
-    /* wait reply or timeout */
-    IBusConnection *connection = ibus_proxy_get_connection ((IBusProxy *) context);
-    while (!ibus_pending_call_get_completed (pending)) {
-        ibus_connection_read_write_dispatch (connection, -1);
-    }
+    call_data = g_slice_new0 (CallData);
+    g_object_ref (context);
+    call_data->context = context;
+    call_data->keyval = keyval;
+    call_data->keycode = keycode;
+    call_data->state = state;
 
-    reply_message = ibus_pending_call_steal_reply (pending);
+    /* set notify callback to handle the reply from daemon */
+    retval = ibus_pending_call_set_notify (pending,
+                                           (IBusPendingCallNotifyFunction)_process_key_event_reply_cb,
+                                           call_data,
+                                           (GDestroyNotify)_call_data_free);
     ibus_pending_call_unref (pending);
 
-    if (reply_message == NULL) {
-        g_debug ("%s: Do not recevie reply of ProcessKeyEvent", DBUS_ERROR_NO_REPLY);
-        retval = FALSE;
+    if (!retval) {
+        _call_data_free (call_data);
+        g_warning ("%s : ProcessKeyEvent", DBUS_ERROR_NO_MEMORY);
+        return FALSE;
     }
-    else if ((error = ibus_error_new_from_message (reply_message)) != NULL) {
-        g_debug ("%s: %s", error->name, error->message);
-        ibus_message_unref (reply_message);
-        ibus_error_free (error);
-        retval = FALSE;
-    }
-    else {
-        if (!ibus_message_get_args (reply_message,
-                                    &error,
-                                    G_TYPE_BOOLEAN, &retval,
-                                    G_TYPE_INVALID)) {
-            g_debug ("%s: %s", error->name, error->message);
-            ibus_error_free (error);
-            retval = FALSE;
-        }
-        ibus_message_unref (reply_message);
-    }
-    return retval;
+
+    return TRUE;
 }
 
 void
