@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,17 +21,28 @@
 /**
  * SECTION: ibusinternal
  * @short_description: IBus DBus setting functions for internal use.
+ * @title: IBusInternal
  * @stability: Stable
- * @see_also: #IBusMainLoop
  *
  * This section contain several IBus house keeping functions.
+ *
+ * @see_also: #IBusMainLoop
+ *
  */
 #ifndef __IBUS_INTERNEL_H_
 #define __IBUS_INTERNEL_H_
 
 #include <glib.h>
-#include <dbus/dbus.h>
+#include "ibusdbus.h"
 
+/**
+ * I_:
+ * @string: A string
+ * @returns: The canonical representation for the string.
+ *
+ * Returns a canonical representation for string.
+ * Interned strings can be compared for equality by comparing the pointers, instead of using strcmp().
+ */
 #define I_(string) g_intern_static_string (string)
 
 /**
