@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* IBus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,8 +21,8 @@
 /**
  * SECTION: ibuskeymap
  * @short_description: Keyboard mapping handling.
+ * @title: IBusKeymap
  * @stability: Stable
- * @see_also: #IBusComponent, #IBusEngineDesc
  *
  * An IBusKeymap defines the mapping between keyboard scancodes and
  * keyboard symbols such as numbers, alphabets, and punctuation marks.
@@ -35,6 +35,8 @@
  * Then ibus_keymap_lookup_keysym() can
  * convert scancodes back to the key symbols.
  *
+ * @see_also: #IBusComponent, #IBusEngineDesc
+ *
  */
 
 #ifndef __IBUS_KEYMAP_H_
@@ -78,7 +80,7 @@ typedef struct _IBusKeymapClass IBusKeymapClass;
  *  5 - shift altgr
  *  6 - numlock
  */
-typedef guint KEYMAP[256][7];
+/* typedef guint KEYMAP[256][7]; */
 
 /**
  * IBusKeymap:
@@ -93,7 +95,7 @@ struct _IBusKeymap {
     /* members */
     /*< public >*/
     gchar *name;
-    KEYMAP keymap;
+    guint keymap[256][7];
 };
 
 struct _IBusKeymapClass {
@@ -105,14 +107,30 @@ GType            ibus_keymap_get_type               (void);
 /**
  * ibus_keymap_new:
  * @name: The keymap file to be loaded, such as 'us', 'jp'.
- * @returns: A newly allocated IBusKeymap; or NULL if failed.
+ * @returns: An IBusKeymap associated with the giving name; or NULL if failed.
  *
- * New an IBusKeymap.
+ * Get an IBusKeymap associated with the giving name.
+ *
+ * This function loads the keymap file specified in @name
+ * in the IBUS_DATA_DIR/keymaps directory.
+ *
+ * Deprecated: This function has been deprecated and should
+ * not be used in newly written code. Please use ibus_keymap_get().
+ */
+IBusKeymap        *ibus_keymap_new                  (const gchar        *name)
+    G_GNUC_DEPRECATED;
+
+/**
+ * ibus_keymap_get:
+ * @name: The keymap file to be loaded, such as 'us', 'jp'.
+ * @returns: An IBusKeymap associated with the giving name; or NULL if failed.
+ *
+ * Get an IBusKeymap associated with the giving name.
  *
  * This function loads the keymap file specified in @name
  * in the IBUS_DATA_DIR/keymaps directory.
  */
-IBusKeymap        *ibus_keymap_new                  (const gchar        *name);
+IBusKeymap        *ibus_keymap_get                  (const gchar        *name);
 
 /**
  * ibus_keymap_lookup_keysym:
