@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,8 +20,9 @@
  */
 /**
  * SECTION: ibusmessage
- * @short_description: A DBusMessage in IBus.
- * @stability: Stable
+ * @Title: IBusMessage
+ * @Short_description: A DBusMessage in IBus.
+ * @Stability: Stable
  *
  * An IBusMessage is essentially a DBusMessage, which representing a message received from or to
  * be sent to another application.
@@ -34,7 +35,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <dbus/dbus.h>
+#include "ibusdbus.h"
 #include "ibuserror.h"
 
 /**
