@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* IBus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,10 +19,9 @@
  * Boston, MA 02111-1307, USA.
  */
 #include "ibusproperty.h"
+#include "ibusproplist.h"
 
 /* functions prototype */
-static void         ibus_property_class_init    (IBusPropertyClass  *klass);
-static void         ibus_property_init          (IBusProperty       *prop);
 static void         ibus_property_destroy       (IBusProperty       *prop);
 static gboolean     ibus_property_serialize     (IBusProperty       *prop,
                                                  IBusMessageIter    *iter);
@@ -31,44 +30,7 @@ static gboolean     ibus_property_deserialize   (IBusProperty       *prop,
 static gboolean     ibus_property_copy          (IBusProperty       *dest,
                                                  const IBusProperty *src);
 
-static void         ibus_prop_list_class_init   (IBusPropListClass  *klass);
-static void         ibus_prop_list_init         (IBusPropList       *prop_list);
-static void         ibus_prop_list_destroy      (IBusPropList       *prop_list);
-static gboolean     ibus_prop_list_serialize    (IBusPropList       *prop_list,
-                                                 IBusMessageIter    *iter);
-static gboolean     ibus_prop_list_deserialize  (IBusPropList       *prop_list,
-                                                 IBusMessageIter    *iter);
-static gboolean     ibus_prop_list_copy         (IBusPropList       *dest,
-                                                 const IBusPropList *src);
-
-static IBusSerializableClass *parent_class = NULL;
-
-GType
-ibus_property_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (IBusPropertyClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_property_class_init,
-        NULL,               /* class finialize */
-        NULL,               /* class data */
-        sizeof (IBusProperty),
-        0,
-        (GInstanceInitFunc) ibus_property_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_SERIALIZABLE,
-                                       "IBusProperty",
-                                       &type_info,
-                                       0);
-    }
-
-    return type;
-}
+G_DEFINE_TYPE (IBusProperty, ibus_property, IBUS_TYPE_SERIALIZABLE)
 
 static void
 ibus_property_class_init (IBusPropertyClass *klass)
@@ -76,8 +38,6 @@ ibus_property_class_init (IBusPropertyClass *klass)
     IBusObjectClass *object_class = IBUS_OBJECT_CLASS (klass);
     IBusSerializableClass *serializable_class = IBUS_SERIALIZABLE_CLASS (klass);
 
-    parent_class = (IBusSerializableClass *) g_type_class_peek_parent (klass);
-
     object_class->destroy = (IBusObjectDestroyFunc) ibus_property_destroy;
 
     serializable_class->serialize   = (IBusSerializableSerializeFunc) ibus_property_serialize;
@@ -126,7 +86,7 @@ ibus_property_destroy (IBusProperty *prop)
         prop->sub_props = NULL;
     }
 
-    IBUS_OBJECT_CLASS (parent_class)->destroy ((IBusObject *)prop);
+    IBUS_OBJECT_CLASS (ibus_property_parent_class)->destroy ((IBusObject *)prop);
 }
 
 gboolean
@@ -135,7 +95,7 @@ ibus_property_serialize (IBusProperty    *prop,
 {
     gboolean retval;
 
-    retval = parent_class->serialize ((IBusSerializable *) prop, iter);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_property_parent_class)->serialize ((IBusSerializable *) prop, iter);
     g_return_val_if_fail (retval, FALSE);
 
     g_return_val_if_fail (IBUS_IS_PROPERTY (prop), FALSE);
@@ -177,7 +137,7 @@ ibus_property_deserialize (IBusProperty    *prop,
     gboolean retval;
     gchar *p;
 
-    retval = parent_class->deserialize ((IBusSerializable *) prop, iter);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_property_parent_class)->deserialize ((IBusSerializable *) prop, iter);
     g_return_val_if_fail (retval, FALSE);
 
     retval = ibus_message_iter_get (iter, G_TYPE_STRING, &p);
@@ -190,6 +150,7 @@ ibus_property_deserialize (IBusProperty    *prop,
     ibus_message_iter_next (iter);
 
     retval = ibus_message_iter_get (iter, IBUS_TYPE_TEXT, &prop->label);
+    g_object_ref_sink (prop->label);
     g_return_val_if_fail (retval, FALSE);
     ibus_message_iter_next (iter);
 
@@ -199,6 +160,7 @@ ibus_property_deserialize (IBusProperty    *prop,
     prop->icon = g_strdup (p);
 
     retval = ibus_message_iter_get (iter, IBUS_TYPE_TEXT, &prop->tooltip);
+    g_object_ref_sink (prop->tooltip);
     g_return_val_if_fail (retval, FALSE);
     ibus_message_iter_next (iter);
 
@@ -215,6 +177,7 @@ ibus_property_deserialize (IBusProperty    *prop,
     ibus_message_iter_next (iter);
 
     retval = ibus_message_iter_get (iter, IBUS_TYPE_PROP_LIST, &prop->sub_props);
+    g_object_ref_sink (prop->sub_props);
     g_return_val_if_fail (retval, FALSE);
     ibus_message_iter_next (iter);
 
@@ -227,7 +190,7 @@ ibus_property_copy (IBusProperty       *dest,
 {
     gboolean retval;
 
-    retval = parent_class->copy ((IBusSerializable *) dest, (IBusSerializable *) src);
+    retval = IBUS_SERIALIZABLE_CLASS (ibus_property_parent_class)->copy ((IBusSerializable *) dest, (IBusSerializable *) src);
     g_return_val_if_fail (retval, FALSE);
 
     g_return_val_if_fail (IBUS_IS_PROPERTY (dest), FALSE);
@@ -305,7 +268,7 @@ ibus_property_set_label (IBusProperty *prop,
         prop->label = ibus_text_new_from_static_string ("");
     }
     else {
-        prop->label = g_object_ref (label);
+        prop->label = g_object_ref_sink (label);
     }
 }
 
@@ -332,9 +295,11 @@ ibus_property_set_tooltip (IBusProperty *prop,
 
     if (tooltip == NULL) {
         prop->tooltip = ibus_text_new_from_static_string ("");
+        g_object_ref_sink (prop->tooltip);
     }
     else {
-        prop->tooltip = g_object_ref (tooltip);
+        prop->tooltip = tooltip;
+        g_object_ref_sink (prop->tooltip);
     }
 }
 
@@ -378,11 +343,13 @@ ibus_property_set_sub_props (IBusProperty *prop,
     }
 
     if (prop_list) {
-        g_object_ref (prop_list);
         prop->sub_props = prop_list;
+        g_object_ref_sink (prop_list);
     }
-    else
+    else {
         prop->sub_props = ibus_prop_list_new ();
+        g_object_ref_sink (prop->sub_props);
+    }
 }
 
 gboolean
@@ -402,12 +369,12 @@ ibus_property_update (IBusProperty *prop,
     if (prop->label) {
         g_object_unref (prop->label);
     }
-    prop->label = (IBusText *) g_object_ref (prop_update->label);
+    prop->label = (IBusText *) g_object_ref_sink (prop_update->label);
 
     if (prop->tooltip) {
         g_object_unref (prop->tooltip);
     }
-    prop->tooltip = (IBusText *) g_object_ref (prop_update->tooltip);
+    prop->tooltip = (IBusText *) g_object_ref_sink (prop_update->tooltip);
     prop->visible = prop_update->visible;
     prop->state = prop_update->state;
     prop->sensitive = prop_update->sensitive;
@@ -415,211 +382,4 @@ ibus_property_update (IBusProperty *prop,
     return TRUE;
 }
 
-GType
-ibus_prop_list_get_type (void)
-{
-    static GType type = 0;
 
-    static const GTypeInfo type_info = {
-        sizeof (IBusPropListClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_prop_list_class_init,
-        NULL,               /* class finialize */
-        NULL,               /* class data */
-        sizeof (IBusProperty),
-        0,
-        (GInstanceInitFunc) ibus_prop_list_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_SERIALIZABLE,
-                                       "IBusPropList",
-                                       &type_info,
-                                       0);
-    }
-
-    return type;
-}
-
-static void
-ibus_prop_list_class_init (IBusPropListClass *klass)
-{
-    IBusObjectClass *object_class = IBUS_OBJECT_CLASS (klass);
-    IBusSerializableClass *serializable_class = IBUS_SERIALIZABLE_CLASS (klass);
-
-    parent_class = (IBusSerializableClass *) g_type_class_peek_parent (klass);
-
-    object_class->destroy = (IBusObjectDestroyFunc) ibus_prop_list_destroy;
-
-    serializable_class->serialize   = (IBusSerializableSerializeFunc) ibus_prop_list_serialize;
-    serializable_class->deserialize = (IBusSerializableDeserializeFunc) ibus_prop_list_deserialize;
-    serializable_class->copy        = (IBusSerializableCopyFunc) ibus_prop_list_copy;
-
-    g_string_append (serializable_class->signature, "av");
-}
-
-static void
-ibus_prop_list_init (IBusPropList *prop_list)
-{
-    prop_list->properties = g_array_new (TRUE, TRUE, sizeof (IBusProperty *));
-}
-
-static void
-ibus_prop_list_destroy (IBusPropList *prop_list)
-{
-    IBusProperty **p;
-    gint i;
-
-    p = (IBusProperty **) g_array_free (prop_list->properties, FALSE);
-
-    for (i = 0; p[i] != NULL; i++) {
-        g_object_unref (p[i]);
-    }
-    g_free (p);
-
-    IBUS_OBJECT_CLASS (parent_class)->destroy ((IBusObject *) prop_list);
-}
-
-static gboolean
-ibus_prop_list_serialize (IBusPropList    *prop_list,
-                          IBusMessageIter *iter)
-{
-    gboolean retval;
-    IBusMessageIter array_iter;
-    IBusProperty *prop;
-    guint i;
-
-    retval = parent_class->serialize ((IBusSerializable *) prop_list, iter);
-    g_return_val_if_fail (retval, FALSE);
-
-    retval = ibus_message_iter_open_container (iter, IBUS_TYPE_ARRAY, "v", &array_iter);
-    g_return_val_if_fail (retval, FALSE);
-
-    i = 0;
-
-    while ((prop = ibus_prop_list_get (prop_list, i)) != NULL) {
-        retval = ibus_message_iter_append (&array_iter, IBUS_TYPE_PROPERTY, &prop);
-        g_return_val_if_fail (retval, FALSE);
-        i ++;
-    }
-
-    retval = ibus_message_iter_close_container (iter, &array_iter);
-    g_return_val_if_fail (retval, FALSE);
-
-    return TRUE;
-}
-
-gboolean
-ibus_prop_list_deserialize (IBusPropList    *prop_list,
-                            IBusMessageIter *iter)
-{
-    gboolean retval;
-    IBusMessageIter array_iter;
-    IBusSerializable *object;
-
-    retval = parent_class->deserialize ((IBusSerializable *) prop_list, iter);
-    g_return_val_if_fail (retval, FALSE);
-
-    g_return_val_if_fail (IBUS_IS_PROP_LIST (prop_list), FALSE);
-
-    retval = ibus_message_iter_recurse (iter, IBUS_TYPE_ARRAY, &array_iter);
-    g_return_val_if_fail (retval, FALSE);
-
-    while (ibus_message_iter_get_arg_type (&array_iter) != G_TYPE_INVALID) {
-        retval = ibus_message_iter_get (&array_iter, IBUS_TYPE_PROPERTY, &object);
-        g_return_val_if_fail (retval, FALSE);
-        ibus_message_iter_next (&array_iter);
-
-        ibus_prop_list_append (prop_list, (IBusProperty *)object);
-        g_object_unref (object);
-    }
-
-    ibus_message_iter_next (iter);
-
-    return TRUE;
-}
-
-
-
-static gboolean
-ibus_prop_list_copy (IBusPropList       *dest,
-                     const IBusPropList *src)
-{
-    gboolean retval;
-    IBusProperty *prop;
-    guint i;
-
-    retval = parent_class->copy ((IBusSerializable *) dest,
-                                 (const IBusSerializable *) src);
-    g_return_val_if_fail (retval, FALSE);
-
-    g_return_val_if_fail (IBUS_IS_PROP_LIST (dest), FALSE);
-    g_return_val_if_fail (IBUS_IS_PROP_LIST (src), FALSE);
-
-    i = 0;
-    while ((prop = ibus_prop_list_get ((IBusPropList *)src, i)) != NULL) {
-        prop = (IBusProperty *) ibus_serializable_copy ((IBusSerializable *) prop);
-        ibus_prop_list_append (dest, prop);
-        g_object_unref (prop);
-        i ++;
-    }
-    return TRUE;
-}
-
-
-IBusPropList *
-ibus_prop_list_new ()
-{
-    IBusPropList *prop_list;
-
-    prop_list = g_object_new (IBUS_TYPE_PROP_LIST, NULL);
-
-    return prop_list;
-}
-
-void
-ibus_prop_list_append (IBusPropList *prop_list,
-                       IBusProperty *prop)
-{
-    g_assert (IBUS_IS_PROP_LIST (prop_list));
-    g_assert (IBUS_IS_PROPERTY (prop));
-
-    g_object_ref (prop);
-
-    g_array_append_val (prop_list->properties, prop);
-}
-
-IBusProperty *
-ibus_prop_list_get (IBusPropList *prop_list,
-                    guint         index)
-{
-    g_assert (IBUS_IS_PROP_LIST (prop_list));
-
-
-    if (index >= prop_list->properties->len)
-        return NULL;
-
-    return g_array_index (prop_list->properties, IBusProperty *, index);
-}
-
-
-
-gboolean
-ibus_prop_list_update_property (IBusPropList *prop_list,
-                                IBusProperty *prop_update)
-{
-    g_assert (IBUS_IS_PROP_LIST (prop_list));
-    g_assert (IBUS_IS_PROPERTY (prop_update));
-
-    gint i;
-
-    for (i = 0; i < prop_list->properties->len; i ++) {
-        IBusProperty *prop = g_array_index (prop_list->properties, IBusProperty *, i);
-        if (ibus_property_update (prop, prop_update)) {
-            return TRUE;
-        }
-    }
-
-    return FALSE;
-}
