@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,7 +33,6 @@
 #ifndef __IBUS_PROXY_H_
 #define __IBUS_PROXY_H_
 
-#include <dbus/dbus.h>
 #include "ibusobject.h"
 #include "ibusconnection.h"
 #include "ibusmessage.h"
@@ -75,8 +74,8 @@ struct _IBusProxyClass {
     IBusObjectClass parent;
 
     /* class members */
-    gboolean    (* ibus_signal)     (IBusProxy   *proxy,
-                                     IBusMessage      *message);
+    gboolean    (* ibus_signal)     (IBusProxy      *proxy,
+                                     IBusMessage    *message);
     /*< private >*/
     /* padding */
     gpointer pdummy[7];
@@ -143,8 +142,8 @@ gboolean         ibus_proxy_call            (IBusProxy          *proxy,
  * Call a method of the corresponding service, and returns an IBusPendingCall used to receive a reply to the message.
  * This function calls ibus_connection_send_with_reply() to do the actual sending.
  *
- * @see_also: ibus_connection_send_with_reply(),
- * @see_also: ibus_proxy_call(), ibus_proxy_send_with_reply(), ibus_proxy_call_with_reply_and_block().
+ * @see_also: ibus_connection_send_with_reply(), ibus_proxy_call(),
+ * ibus_proxy_send_with_reply(), ibus_proxy_call_with_reply_and_block().
  */
 gboolean         ibus_proxy_call_with_reply (IBusProxy          *proxy,
                                              const gchar        *method,
@@ -166,10 +165,11 @@ gboolean         ibus_proxy_call_with_reply (IBusProxy          *proxy,
  *
  * Call a method of the corresponding service and blocks a certain time period while waiting for
  * an IBusMessage as reply.
- * If the IBusMessagwe is not NULL,  it calls ibus_connection_send_with_reply_and_block() to do the actual sending.
+ * If the IBusMessage is not NULL, it calls ibus_connection_send_with_reply_and_block() to do the
+ * actual sending.
  *
- * @see_also: ibus_connection_send_with_reply_and_block(),
- * @see_also: ibus_proxy_call(), ibus_proxy_send_with_reply(), ibus_proxy_call_with_reply_and_block().
+ * @see_also: ibus_connection_send_with_reply_and_block(), ibus_proxy_call(), 
+ * ibus_proxy_send_with_reply(), ibus_proxy_call_with_reply_and_block().
  */
 IBusMessage     *ibus_proxy_call_with_reply_and_block
                                             (IBusProxy          *proxy,
@@ -191,8 +191,8 @@ IBusMessage     *ibus_proxy_call_with_reply_and_block
  * an IBusPendingCall used to receive a reply to the message.
  * This function calls ibus_connection_send_with_reply() to do the actual sending.
  *
- * @see_also: ibus_connection_send_with_reply(),
- * @see_also: ibus_proxy_send(), ibus_proxy_call_with_reply(), ibus_proxy_send_with_reply_and_block().
+ * @see_also: ibus_connection_send_with_reply(), ibus_proxy_send(),
+ * ibus_proxy_call_with_reply(), ibus_proxy_send_with_reply_and_block().
  */
 gboolean         ibus_proxy_send_with_reply (IBusProxy          *proxy,
                                              IBusMessage        *message,
@@ -207,10 +207,11 @@ gboolean         ibus_proxy_send_with_reply (IBusProxy          *proxy,
  *
  * Send an IBusMessage to the corresponding service and blocks a certain time period while waiting for
  * an IBusMessage as reply.
- * If the IBusMessage is not NULL,  it calls ibus_connection_send_with_reply_and_block() to do the actual sending.
+ * If the IBusMessage is not NULL, it calls ibus_connection_send_with_reply_and_block() to do the
+ * actual sending.
  *
- * @see_also: ibus_connection_send_with_reply_and_block(),
- * @see_also: ibus_proxy_send(), ibus_proxy_send_with_reply(), ibus_proxy_call_with_reply_and_block().
+ * @see_also: ibus_connection_send_with_reply_and_block(), ibus_proxy_send(), ibus_proxy_send_with_reply(), 
+ * ibus_proxy_call_with_reply_and_block().
  */
 IBusMessage     *ibus_proxy_send_with_reply_and_block
                                             (IBusProxy          *proxy,
@@ -275,7 +276,7 @@ const gchar     *ibus_proxy_get_interface   (IBusProxy          *proxy);
 /**
  * ibus_proxy_get_connection:
  * @proxy: An IBusProxy.
- * @returns: The connection of the proxy object.
+ * @returns: (transfer none): The connection of the proxy object.
  *
  * Get the connection of a proxy object.
  */
