@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,6 @@
 #ifndef __IBUS_SERVICE_H_
 #define __IBUS_SERVICE_H_
 
-#include <dbus/dbus.h>
 #include "ibusobject.h"
 #include "ibusconnection.h"
 
@@ -66,9 +65,28 @@ struct _IBusService {
     /* instance members */
 };
 
+/**
+ * ServiceIBusMessageFunc:
+ * @service: An IBsService.
+ * @connection: Connection to IBus daemon.
+ * @message: IBusMessage to be sent.
+ * @returns: %TRUE if succeed; %FALSE if failed.
+ *
+ * Prototype of IBus service message sending callback function.
+ */
 typedef gboolean  (* ServiceIBusMessageFunc)    (IBusService    *service,
                                                  IBusConnection *connection,
                                                  IBusMessage    *message);
+
+/**
+ * ServiceIBusSignalFunc:
+ * @service: An IBsService.
+ * @connection: Connection to IBus daemon.
+ * @message: IBusMessage to be sent.
+ * @returns: %TRUE if succeed; %FALSE if failed.
+ *
+ * Prototype of IBus service signal sending callback function.
+ */
 typedef gboolean  (* ServiceIBusSignalFunc)     (IBusService    *service,
                                                  IBusConnection *connection,
                                                  IBusMessage    *message);
@@ -137,10 +155,9 @@ gboolean         ibus_service_add_to_connection (IBusService    *service,
 /**
  * ibus_service_get_connections:
  * @service: An IBusService.
- * @returns: A newly allocated list of connections.
+ * @returns: (transfer container) (element-type IBusConnection): A newly allocated list of connections.
  *
- * Returns a copy of list of connections.
- * List elements need to be unref by g_object_unref().
+ * Returns a copy of list of connections, but the caller does not own the element.
  */
 GList           *ibus_service_get_connections   (IBusService    *service);
 
