@@ -1,7 +1,7 @@
 /* vim:set et sts=4: */
 /* bus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -265,7 +265,6 @@ ibus_xml_parse_buffer (const gchar *buffer)
 }
 
 
-void     ibus_xml_free          (XMLNode        *node);
 static void
 output_indent (int level, GString *output)
 {
@@ -311,7 +310,7 @@ xml_output_indent (const XMLNode *node, int level, GString *output)
 }
 
 void
-xml_output (const XMLNode *node, GString *output)
+ibus_xml_output (const XMLNode *node, GString *output)
 {
     xml_output_indent (node, 0, output);
 }
